/*
   apa.c -- APA 
   Copyright 2007 by David Laine, david dot laine a free o fr

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
   
 */

#include <stdlib.h>
#include <stdio.h>

//#include <tiff.h>
#include <tiffio.h>

#include <time.h>


#include <glib.h>
#include <gmodule.h>

#include "api.h"

const char * A_TYPE[4] = 
  { "Entier",
    "Reel",
    "Chaine de caracters",
    "Poucentage"
  };


typedef int  (* Fonction)(struct image *,
			  struct image * ,
			  int , int ,
			  int , int ,
			  struct parametre * );

typedef struct filtre * (* A_Name)();
typedef struct parametre  * (* A_Init)();


int ApplicationFiltre(char *nomFiltre,
		      char *imSource,
		      char *imDest,
		      int X,int Y,int L,int H,
		      struct parametre *p)
{
  GModule*module;
  GError **error;
  
  if (!g_module_supported())
    {
      printf ("Plugins non supports\n");
      exit(1);
    }
  gchar * filename = g_module_build_path (NULL,nomFiltre);
    
  module = g_module_open (filename, G_MODULE_BIND_LAZY);
  if (!module)
    {
      printf ("Imposible de charger %s\n",filename);
      exit(1);
    }
 
  Fonction  fonction;
  
  if (!g_module_symbol (module, "Do" , (gpointer *) &fonction))
    {
      printf("Ne trouve pas la fonction Do dans la lib!\n",nomFiltre);
      exit(1);
    }
  
  // Charger une image

  struct image in ;
  //in=malloc(sizeof(struct image));
  loadTiff(imSource,&in,A_TRUE);

  struct image out ;  
  //out=malloc(sizeof(struct image));
  copyImage(&in, &out);
  
 
  if((out.buffer = (char *) malloc(out.bufferSize)) == NULL){
    fprintf(stderr, "Could not allocate enough memory for the uncompressed image\n");
    exit(42);
  }
  
  unsigned long i;
  for (i=0;i<out.bufferSize;i++)
    out.buffer[i]=in.buffer[i];
  
  // Appliquer le plugin



  // Initialisation par dfaut
  X = X<0 ? 0: X>in.width ? in.width : X;
  Y = Y<0 ? 0: Y>in.height ? in.height : Y;
  L = L==-1? (in.width-X): L> (in.width-X)?(in.width-X):L ;
  H = H==-1? (in.height-Y): H>(in.height-Y)?(in.height-Y):H;
  

  //fonction(&in,&out,0,0,1541,2*in.height/3,&p);
  //p.floatvalue=0.002;
  //fonction(&in,&out,1542,0,(in.width-1542),2*in.height/3,&p);
  fonction(&in,&out,X,Y,L,H,p);
    
  // Enregistrer l'image

  writeTiff(&out,imDest);

}

struct parametre * analyseParam(char *txt)
{
  struct parametre * p;
  // On suppose un parametre
  p=malloc(sizeof(struct parametre));
  p->type = INT;
  p->intvalue=atoi(txt);
  p->floatvalue=0.0f;
  p->next=NULL;
  return p;
}
/* int analysePoint(char *txt, int *a,int *b) */
/* { */
/*   int i; */
/*   int l = strlen(txt); */
/*   int sep=-1; */
/*   int err=0; */
/*   for (i=0;i<l;i++) */
/*     switch (txt[i]) */
/*       { */
/*       case 'x': */
/*       case 'X': */
/* 	sep=i; */
/* 	break; */
/*       case '0': */
/*       case '1': */
/*       case '2': */
/*       case '3': */
/*       case '4': */
/*       case '5': */
/*       case '6': */
/*       case '7': */
/*       case '8': */
/*       case '9': */
/* 	break; */
/*       default: */
/* 	err=1; */
/* 	break; */
/*       } */
/*   if (err) */
/*     return 0; */
/*   *b=atoi(&txt[sep+1]); */
/*   txt[sep]='\0'; */
/*   *a=atoi(txt); */

/*   return 1; */
/* } */

int LectureListeFiltre()
{

  GModule*module;
  GError **error;
  A_Name name;
  A_Init init;
  
  if (!g_module_supported())
    {
      printf ("Plugins non supports\n");
      exit(1);
    }
  
  // Lecture du repertoire courant
  GDir *dir = g_dir_open (PLUGINS, 0, NULL);
  printf("----------------------------\n");
  printf("List des filtres disponible \n");
  printf("----------------------------\n");fflush(stdout);

  if (dir)
    {
      const gchar *nomFiltre = NULL;

      while (nomFiltre = g_dir_read_name (dir))
	{
	  gchar * filename = g_module_build_path (NULL,nomFiltre);
	  module = g_module_open (filename, G_MODULE_BIND_LAZY);
	  if (module)
	    {
	      Fonction  fonction;
  
	      if (g_module_symbol (module, "Name" , (gpointer *) &name))
		{
		  struct filtre *f;
		  f=name();
		  printf("\n<<%s>> dans %s\n\t%s\n",
			 f->name,filename,
			 f->desc);fflush(stdout);
		  
		  if (g_module_symbol (module, "Init" , (gpointer *) &init))

		    {
		      struct parametre *p;
		      p=init();
		      while (p!=NULL)
			{
			  
			  printf("Parametres : %s de type %s \n\t %s \n",
				 p->name,
				 A_TYPE[p->type],
				 p->desc);
			  p=p->next;
			}
		    }
		}
	      g_module_close (module);
	    }
	}
      g_dir_close (dir), dir = NULL;
    }  
}


int usage(char *pgm)
{
  wchar_t *s = L"[OPTION] FILTRE SOURCE DESTINATION\n"
    L"\nCe logiciel permet d'appliquer un filtre a une image tiff.\n"
    L"Le filtre FILTRE s'aplique sur l'image SOURCE a partir du point A.\n"
    L"Le point A est par defaut a l'origine.Le filtre est applique sur un\n"
    L"rectangle definit par l'option T qui donne la largeur et la hauteur\n"
    L"ou par un point B qui est par defaut la largeur et la hauteur de \n"
    L"l'image source\n"
    L"\n\n"
     L"  -A 0x0    \t Point A marquant le debut d'application du filtre\n" 
    L"  -B largxhaut    \t Point B marquant la fin d'application du filtre\n"
    L"  -T largxhaut    \t Largeur et hauteur d'application du filtre \n"
    L"  -l     \t Liste des filtres disponibles\n"
    L"  -p     \t Parametres separes par une virgule ex: -p 12,500\n"
    L"  -?     \t this help\n";
  printf("Usage: %s %ls\n",pgm,s);
  return 1;
}

int main(int argc, char* argv[])
{
  int c;
  extern int optind;
  extern char* optarg;
  clock_t start, end;
  double elapsed;

  int X,Y,L,H;
  int X1,Y1;
  X=-1;
  Y=-1;
  L=-1;
  H=-1;

  if (argc ==1) return usage(argv[0]);
  
  start = clock();	/* Lancement de la mesure */

  struct parametre *p ;
  p=NULL;
  while ((c = getopt(argc, argv, "A:B:T:lp:?")) != -1)
    
    
    switch (c) 
      {
      case 'p':
	p=analyseParam(optarg);
	if (p==NULL)
	  {
	    printf("Pb de reconnaisance du point A %s. Le format est 12x48 par exemple.\n", optarg);
	    exit(1);
	  }
	break;
      case 'A':
	if (!analysePoint(optarg,&X,&Y))
	  {
	    printf("Pb de reconnaisance du point A %s. Le format est 12x48 par exemple.\n", optarg);
	    exit(1);
	  }
	break;
      case 'B':
	if (!analysePoint(optarg,&X1,&Y1))
	  {
	    printf("Pb de reconnaisance du point B %s. Le format est 12x48 par exemple.\n", optarg);
	    exit(1);
	  }
	if (X1>X) L=X1-X;
	if (Y1>Y) H=Y1-Y;
	
	break;
      case 'T':
	if (!analysePoint(optarg,&L,&H))
	  {
	    printf("Pb de reconnaisance des largeur hauteur %s. Le format est 12x48 par exemple.\n", optarg);
	    exit(1);
	  }
	break;
      case 'l':
	LectureListeFiltre();
	exit(1);
	break;
      case '?':
      default :
	usage(argv[0]);
	break;
      }

  int i;

  if (argc!=(optind+3)) return usage(argv[0]);
  
  for (i=optind;i<argc;i++)
    printf("arg %s\n",argv[i]);


  ApplicationFiltre(argv[optind],argv[optind+1],argv[optind+2],
		    X,Y,L,H,p);
  
  end = clock();		/* Arret de la mesure */
  elapsed = ((double)end - start) / CLOCKS_PER_SEC; /* Conversion en seconde */
  printf("Temps execution %f\n",elapsed);

  return 0;

}
