#include <stdlib.h>
#include <stdio.h>
#include <tiffio.h>
#include <tiff.h>
#include <math.h>

#include "api.h"

char * Version()
{
  // Version de l'api utilise
  return "0.9";
}


struct filtre *  Name()
{
  struct filtre *f;
  f=malloc(sizeof(struct filtre));
  f->name="FlouGaussienConv";
  f->desc="Flou Gaussien par convolution";
  return f;
}

struct parametre * Init()
{

  struct parametre *p2;
  p2=malloc(sizeof(struct parametre));
  p2->name="sigma";
  p2->desc="Sigma";
  p2->type=FLOAT;
  p2->intvalue=0;
  p2->floatvalue=1.0;
  p2->next=NULL;
  
  struct parametre *p1;
  p1=malloc(sizeof(struct parametre));
  p1->name="largeur";
  p1->desc="rayon de la matrice de convolution (par defaut 1)";
  p1->type=INT;
  p1->intvalue=1;
  p1->floatvalue=0.0;
  p1->next=p2;

  

  return p1;
}

float * creerNoyauGaussien(int rayon, float sigma)
{
  /*
    kernel : Noyau de convolution de taille 
    largeur = 2*rayon+1
    hauteur = 2*rayon+1
    et centr en (rayon, rayon)
  */
  float *kernel;
  kernel = malloc(sizeof(int)*(2*rayon+1)*(2*rayon+1));
  
  if (sigma == 0) return NULL;
  if (rayon<1) return NULL;
  float gaussianKernelFactor =0;
  float e=0;
  
  int ky,kx;
  unsigned int p;
  for (ky=-rayon;ky<(rayon+1);ky++)
    {
      for (kx=-rayon;kx<(rayon+1);kx++)
	{
	  e =  exp( - (kx*kx+ky*ky) / (2*sigma*sigma));
	  gaussianKernelFactor =  gaussianKernelFactor  + e;
	  p = (kx+rayon)*(2*rayon+1)+ky+rayon;
	  kernel[p]= e  ;
	  //printf("%f ",kernel[p]);
	    }
      //printf("\n");
    }
  //fflush(stdout);
  
  /*on divise tout par le facteur afin que la somme de tous les lments
   * soient gaux  1
   */
  //printf("%f\n",kx,ky,gaussianKernelFactor);fflush(stdout);
  for (ky=-rayon;ky<(rayon+1);ky++)
    for (kx=-rayon;kx<(rayon+1);kx++)
      {
	p = (kx+rayon)*(2*rayon+1)+ky+rayon;
	kernel[p]/= gaussianKernelFactor ;
      }
  
  return kernel;
}


int  Do(struct image * source, 
	struct image * destination,
	int X, int Y,
	int Larg, int Haut,
	struct parametre * p)
{
 
  printf("Flou Gaussien par Conv %i,%i,%i\n",
	 source->sampleParPixel,source->bps,Haut);fflush(stdout);
  // On calcul une fonction z=a+bx+cy qui interpole l'image
  
  // Recherche de la fonction de recherche d'une valeur d'un pixel
  _ImVal ImVal  = DefImVal(source);
  // Recherche de la fonction de maj d'une valeur d'un pixel
  _ImMaj ImMaj  = DefImMaj(destination);
  

  int rayon=1;
  float sigma=1;
  struct parametre *point;
  point=p;
  while (point!=NULL)
    {
      if (strcmp(point->name,"largeur")==0)
	rayon=point->intvalue;
      if (strcmp(point->name,"sigma")==0)
	sigma=point->floatvalue;
      point=point->next;
    }
  printf("parametre %i %f\n",rayon, sigma);fflush(stdout);
  float *kernel;
  kernel = creerNoyauGaussien(rayon, sigma);
  int ky,kx;
  unsigned int pp;

  printf("ok pour le rayon %i et sigma %f\n",rayon, sigma);fflush(stdout);
  
  unsigned long x,y,count;
  // Que pour des images RGB en 16 bits pour l'instant
  uint16 R,G,B;
  float cR,cG,cB;
  // En fonction de la taille de la matrice, on dcale la fenetre d'application du filtre
  Larg = (X+Larg+rayon+1)>source->width?Larg-rayon-1:Larg;
  Haut = (Y+Haut+rayon+1)>source->height?Haut-rayon-1:Haut;
  X = X<rayon?rayon:X;
  Y = Y<rayon?rayon:Y;

  printf("xy lw %i %i %i %i \n",X,Y,Larg,Haut);fflush(stdout);

  // lire l'image
  if (source->sampleParPixel==3)
    if (source->bps==16)
      {  
	
	for (y=Y;y<(Y+Haut);y++)
	  for (x=X;x<(X+Larg);x++)
	    {

	      cR =0.0f;
	      cG =0.0f; 
	      cB =0.0f;
	      // Appication de la matrice
	      for (ky=-rayon;ky<(rayon+1);ky++)
		for (kx=-rayon;kx<(rayon+1);kx++)
		  {

		    (ImVal)(source,x+kx,y+ky,&R,&G,&B);
		    pp = (kx+rayon)*(2*rayon+1)+ky+rayon;
		    cR+=R*kernel[pp];
		    cG+=G*kernel[pp];
		    cB+=B*kernel[pp];

/* 		    if ((x==X)&&(y==Y)) */
/* 		      { */
/* 			printf("(%i %i)-> %i %f %f\n",x+kx,y+ky,R,cR,kernel[pp]);fflush(stdout); */
/* 		      } */

		  }

	      // Mise a jour du pixel
	      //count = y*source->width+ x;

	      R=cR<0?0:cR>65535?65335:(int)cR;
	      G=cG<0?0:cG>65535?65335:(int)cG;
	      B=cB<0?0:cB>65535?65335:(int)cB;

	      R=255;
	      G=0;
		B=0;
	      (ImMaj)(destination,x,y,R,G,B);

/* 	      destination->buffer[count*6+1]=R/256; */
/* 	      destination->buffer[count*6]=R */
/* 		-256*destination->buffer[count*6+1]; */
/* 	      destination->buffer[count*6+3]=G/256; */
/* 	      destination->buffer[count*6+2]=G */
/* 		-256*destination->buffer[count*6+3]; */
	      
/* 	      destination->buffer[count*6+5]=B/256; */
/* 	      destination->buffer[count*6+4]=B */
/* 		-256*destination->buffer[count*6+5]; */

/* 	      if ((x==X)&&(y==Y)) */
/* 		{ */
/* 		  printf("soit (%i,%i) %f %i\n",x,y,cR,R);fflush(stdout); */
/* 		} */

	    }
      }
}

  


