#include <stdlib.h>
#include <stdio.h>
#include <fftw3.h>
#include <math.h>

#include "api.h"


struct filtre *  Name()
{
  struct filtre *f;
  f=malloc(sizeof(struct filtre));
  f->name="FlouGaussien";
  f->desc="Applique un flou gaussien";
  return f;
}

struct parametre * Init()
{
  struct parametre *p;
  p=malloc(sizeof(struct parametre));
  p->name="sigma";
  p->desc="Do";
  p->type=FLOAT;
  p->intvalue=0;
  p->floatvalue=0.0002;
  p->next=NULL;
  return p;
}

/* in : image d'entre
 * reOut : partie relle de l'image de sortie
 * imOut : partie imaginaire de l'image de sortie
 * largeur : largeur des images d'entre et de sortie
 * hauteur : idem
 */
void fourierForward(const float* in,
                    float* reOut,
                    float* imOut,
                    unsigned int largeur,
                    unsigned int hauteur)
{
   fftw_complex* spatial_repr;
   fftw_complex* frequency_repr;
   unsigned int i;
   unsigned int j;
   fftw_plan plan;
   int x,y;

   spatial_repr= fftw_malloc(sizeof(fftw_complex)*largeur*hauteur);
   frequency_repr= fftw_malloc(sizeof(fftw_complex)*largeur*hauteur);

   /*On remplit la structure qui sera utilise par fftw*/
   for(i=0;i<largeur*hauteur;i++)
   {
      spatial_repr[i][0] = in[i];
      spatial_repr[i][1] =  0.0f;
   }

   /*on calcule le plan d'excution*/
   plan=fftw_plan_dft_2d(hauteur, largeur, spatial_repr, frequency_repr, FFTW_FORWARD, FFTW_ESTIMATE);
   
   /*on effectue la transforme de Fourier*/
    fftw_execute(plan);
   /*on retranscrit le rsultat en 2 images, l'une reprsentant la partie relle, l'autre
     la partie imaginaire*/
    for(j=0;j<hauteur;j++)
      for(i=0;i<largeur;i++)
	{
	  /*on recentre l'image*/
	  x=i;
	  y=j;
	  if (i<largeur/2 && j<hauteur/2){ x=i+largeur/2; y=j+hauteur/2; }
          if (i>=largeur/2 && j<hauteur/2){ x=i-largeur/2; y=j+hauteur/2; }
          if (i<largeur/2 && j>=hauteur/2){ x=i+largeur/2; y=j-hauteur/2; }
          if (i>=largeur/2 && j>=hauteur/2){ x=i-largeur/2; y=j-hauteur/2; }
          reOut[y*largeur+x]=frequency_repr[j*largeur+i][0];
          imOut[y*largeur+x]=frequency_repr[j*largeur+i][1];
      }

/*    for(i=0;i<largeur*hauteur;i++) */
/*       { */
/*           reOut[i]=frequency_repr[i][0]; */
/*           imOut[i]=frequency_repr[i][1]; */
/*       } */
   
   /*on dtruit les objets*/
   fftw_destroy_plan(plan);
   fftw_free(spatial_repr);
   fftw_free(frequency_repr);
}

 
/* reIn : partie rel de l'image dans l'espace de Fourier
 * imIn : partie imaginaire de l'image dans l'espace de Fourier
 * out : image de sortie
 * largeur : largeur des images d'entre et de sortie
 */
void fourierBackward(const float* reIn,
                     const float* imIn,
                     float* out,
                     unsigned int largeur,
                     unsigned int hauteur)
{
   fftw_complex* spatial_repr;
   fftw_complex* frequency_repr;
   unsigned int i;
   unsigned int j;
   int x,y;
   fftw_plan plan;

   spatial_repr= fftw_malloc(sizeof(fftw_complex)*largeur*hauteur);
   frequency_repr= fftw_malloc(sizeof(fftw_complex)*largeur*hauteur);

   for(j=0;j<hauteur;j++)
     for(i=0;i<largeur;i++)
       {
	 /*on dcentre*/
	 x=i;
	 y=j;
	 if (i<largeur/2 && j<hauteur/2){ x=i+largeur/2; y=j+hauteur/2; }
	 if (i>=largeur/2 && j<hauteur/2){ x=i-largeur/2; y=j+hauteur/2; }
	 if (i<largeur/2 && j>=hauteur/2){ x=i+largeur/2; y=j-hauteur/2; }
	 if (i>=largeur/2 && j>=hauteur/2){ x=i-largeur/2; y=j-hauteur/2; }
	 frequency_repr[j*largeur+i][0]=reIn[y*largeur+x];
	 frequency_repr[j*largeur+i][1]=imIn[y*largeur+x];
       }
   
/*    for(i=0;i<largeur*hauteur;i++) */
/*      //for(j=0;j<hauteur;j++) */
/*      //for(i=0;i<largeur;i++) */
/*      { */
       
/*        frequency_repr[i][0]=reIn[i]; */
/*        frequency_repr[i][1]=imIn[i]; */
/*      } */

  plan=fftw_plan_dft_2d(hauteur, largeur, frequency_repr, spatial_repr, FFTW_BACKWARD, FFTW_ESTIMATE);

  fftw_execute(plan);

   /*on retranscrit l'image complexe en image relle, sans oublier de diviser par largeur*hauteur*/
   for(i=0;i<largeur*hauteur;i++)
   {
      out[i]=spatial_repr[i][0]/(largeur*hauteur);
   }

   fftw_destroy_plan(plan);
   fftw_free(spatial_repr);
   fftw_free(frequency_repr);
}

		


int  Do(struct image * source, 
	struct image * destination,
	int X, int Y,
	int Larg, int Haut,
	struct parametre * p)
{
  float sigma=0.02;
  if (p!=NULL)
    sigma=p->floatvalue;

  printf("Flou %f %i,%i,%i\n",sigma,source->sampleParPixel,source->bps,Haut);

  if (source->sampleParPixel==3)
    if (source->bps==16)
      {
	

	// Transorm de Fourrier
	
	float* in;
	float* out;
	int x,y;
	int R;
	in = malloc(sizeof(float)*Larg*Haut);
	uint32 i,count,j;
	i=0;
	for (y=Y;y<(Y+Haut);y++)
	  for (x=X;x<(X+Larg);x++)
	    {
	      // Rouge
	      count = y*source->width+ x;
	      in[i]=source->buffer[count*6  ]+256*source->buffer[count*6+1];

	      //printf(" %ix%i R %f\n",x,y,in[i]);

	      i++;
	      
	    }
	float* reOut;
	float* imOut;
	reOut = malloc(sizeof(float)*Larg*Haut);
	imOut = malloc(sizeof(float)*Larg*Haut);
	fourierForward(in,
		       reOut,
		       imOut,
		       (unsigned int)Larg,
		       (unsigned int)Haut);

	// Edition
/* 	for(j=0; j<Haut;j++)  */
/*  	  for(i=0; i<Larg;i++) */
/* 	    { */
/* 	      printf("%ix%i re %f im %f\n",i,j, */
/* 		     reOut[i + Larg*j], */
/* 		     imOut[i + Larg*j]); */
/* 	    } */
	

	//Traitement Gaussien
	
	float* reIn;
	float* imIn;
	reIn = malloc(sizeof(float)*Larg*Haut);
	imIn = malloc(sizeof(float)*Larg*Haut);
	
	float a,b,c,d;
	
	float r = sigma*sigma/2.0;
	printf("Sigma et r %f %f\n",sigma,r);

	for(j=0; j<Haut;j++) 
 	  for(i=0; i<Larg;i++)
	    {
	      reIn[i + Larg*j]=reOut[i + Larg*j]; 
	      imIn[i + Larg*j]=imOut[i + Larg*j]; 
/* 	      a=reOut[i + Larg*j]; */
/* 	      b=imOut[i + Larg*j]; */

/* 	      c=exp(-r*((i-Larg/2.0)*(i-Larg/2.0) + */
/* 			(j-Haut/2.0)*(j-Haut/2.0))); */

	      
/* 	      d=0.0f; */
/* 	      reIn[i + Larg*j]=a*c - b*d; */
/* 	      imIn[i + Larg*j]=b*c + a*d; */
	      
/* 	      if ((i==1) && (j==1)) */
/* 		printf("Valeur %f %f %f %f %f %f %f\n",c, */
/* 		       reIn[i + Larg*j],imIn[i + Larg*j], */
/* 		       reOut[i + Larg*j],imOut[i + Larg*j], */
/* 		       a,b */
/* 		       ); */
		       
	      
	    }	

	// Transform de fourrier inverse
	out = malloc(sizeof(float)*Larg*Haut);
	
	
/* 	fourierBackward(reIn, */
/* 			imIn, */
/* 			out, */
/* 			(unsigned int)Larg, */
/* 			(unsigned int)Haut); */
	
	i=0;
	for (y=Y;y<(Y+Haut);y++)
	  for (x=X;x<(X+Larg);x++)
	    {
	      // Rouge
	      count = y*destination->width+ x;
	      if (reIn[i]<0) //(out[i]<0)
		{
		  destination->buffer[count*6+1]=0;
		  destination->buffer[count*6]=0;
		}
	      else
		if (reIn[i]>65535) //(out[i]>65535)
		  {
		    destination->buffer[count*6+1]=255;
		    destination->buffer[count*6]=255;
		  }
		else
		  {
		    destination->buffer[count*6+1]=reIn[i]/256;
		    destination->buffer[count*6] = reIn[i]-destination->buffer[count*6+1];
/* 		    destination->buffer[count*6+1]=out[i]/256; */
/* 		    destination->buffer[count*6] = out[i]-destination->buffer[count*6+1]; */
		  }
	     
	      
	      // Autres couleurs
	      destination->buffer[count*6+2] = 0;
	      destination->buffer[count*6+3]=0;
	      destination->buffer[count*6+4] = 0;
	      destination->buffer[count*6+5]=0;
	      i++;
	    }
      }
}
