/*
   apa.c -- APA 
   Copyright 2007 by David Laine, david dot laine a free o fr

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
   
 */

#include "apa.h"


const char * REPERTOIRES[LEN_REPERTOIRES] = 
{ "offsets",
  "lights",
  "darks",
  "flats",
  "results"
};

int min(int a, int b)
{
  if (a<b) return a;
  return b;
}

int max(int a, int b)
{
  if (a>b) return a;
  return b;
}

int makeRep()
{
  int i;
  for (i=0; i<LEN_REPERTOIRES;i++)
    {
      if (mkdir(REPERTOIRES[i])!=0)
	printf("Impossible de creer le repertoire %s (il est possible qu'il existe deja...\n",REPERTOIRES[i]);
    }
}

void findSizeImage(struct image * im)
{

  /* Try to find the zize of a image */
  /* We try with the size of a image of LIGHTS directpry */
  
  struct dirent *lecture;
  DIR *rep;
  rep = opendir(REPERTOIRES[LIGHTS]);
  
  int i=0;

  /* lire les images */
  while ((lecture = readdir(rep))) {
    /* On prend que les fichiers .tiff */
    if  (strlen(lecture->d_name) > 5 )
	 
      {
	char *p = lecture->d_name;
	p = p + strlen(lecture->d_name)-5;
	if (strcmp(p, ".tiff")==0)
	  {
	    char nTiff[256];
	    sprintf(nTiff,"%s//%s",REPERTOIRES[LIGHTS],lecture->d_name);

	    TIFF* tif = TIFFOpen(nTiff, "r");
	    if (tif) {
	      TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &im->width);
	      TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &im->height);
	      //printf("findSizeImage %ix%i",im->width,im->height);
      
	      TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &im->bps);
	      TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &im->sampleParPixel);

	      closedir(rep);
	      TIFFClose(tif);
	      return;
	    }

	  }
      }
  }
  closedir(rep);
 
  im->width = 0;
  im->height =  0;
  printf("Probleme : Pas de taille trouvee !");
  exit(-1);
  
}

int writeTiff(struct image *im, char *nFichier)
{
  // Define an image
  TIFF *image;

  // Open the TIFF file
  if((image = TIFFOpen(nFichier, "w")) == NULL){
    printf("Could not open %s for writing\n", nFichier);
    exit(42);
  }

  // We need to set some values for basic tags before we can add any data
  TIFFSetField(image, TIFFTAG_IMAGEWIDTH, im->width);
  TIFFSetField(image, TIFFTAG_IMAGELENGTH, im->height);
  TIFFSetField(image, TIFFTAG_BITSPERSAMPLE, im->bps);
  
  TIFFSetField(image, TIFFTAG_SAMPLESPERPIXEL,im->sampleParPixel);
  TIFFSetField(image, TIFFTAG_ROWSPERSTRIP,im->height );

  /* A liitle compression */
  TIFFSetField(image, TIFFTAG_COMPRESSION, COMPRESSION_LZW);

  if (im->sampleParPixel==1)
    TIFFSetField(image, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
  else
    TIFFSetField(image, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);

  //TIFFSetField(image, TIFFTAG_FILLORDER, FILLORDER_MSB2LSB);
  TIFFSetField(image, TIFFTAG_PLANARCONFIG,1);// PLANARCONFIG_CONTIG);
  
  // Write the information to the file
  TIFFWriteEncodedStrip(image, 0, im->buffer, im->bufferSize);

  /* TODO : how to save exposure ? */
/*   if (im->exposure!=0) */
/*     { */
  
/*       TIFFSetField(image, EXIFTAG_EXPOSURETIME, im->exposure); */
/*     } */

  // Close the file
  TIFFClose(image);
}


int loadTiff(char *nTiff,struct image * im,int needBuffer)
{
    TIFF* tif = TIFFOpen(nTiff, "r");
    if (tif) {
	// Read in the possibly multiple strips
	tsize_t stripSize;
	int stripMax, stripCount;
	unsigned long imageOffset, result;
	
	//uint16 photo, bps, spp, fillorder;
	TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &im->width);
	TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &im->height);
	TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &im->bps);
	TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &im->sampleParPixel);

	stripSize = TIFFStripSize (tif);
	stripMax = TIFFNumberOfStrips (tif);
	imageOffset = 0;
	im->bufferSize =  stripMax * stripSize;
	if (needBuffer)
	  {
	    if((im->buffer = (char *) malloc(im->bufferSize)) == NULL){
	      fprintf(stderr, "Could not allocate enough memory for the uncompressed image\n");
	      exit(42);
	    }
	    for (stripCount = 0; stripCount < stripMax; stripCount++){
	      if((result = TIFFReadEncodedStrip (tif, stripCount,
						 im->buffer + imageOffset,
						 stripSize)) == -1){
		fprintf(stderr, "Read error on input strip number %d\n", stripCount);
		exit(42);
	      }
	      
	      imageOffset += result;
	    }
	
	    // Deal with fillorder
	    /* 	if(TIFFGetField(tif, TIFFTAG_FILLORDER, &fillorder) == 0){ */
	    /* 	  printf( "Image has an undefined fillorder\n"); */
	    /* 	} */
	  }
	uint32 exif;
	im->exposure=0;
	if (TIFFGetField(tif, TIFFTAG_EXIFIFD, &exif))
	  {
	    TIFFReadEXIFDirectory(tif, exif);

	    //void * isoo;
	    //if (TIFFGetField(tif, EXIFTAG_ISOSPEEDRATINGS, isoo))
	    //  printf("iso %i \n",12);
	    
	    const TIFFFieldInfo *fip;
	    fip = TIFFFieldWithTag(tif, EXIFTAG_EXPOSURETIME);
	    if(fip != NULL)
	      {
		void *raw_data;
		raw_data = _TIFFmalloc(_TIFFDataSize(fip->field_type)
				       * fip->field_readcount);
		
		if (TIFFGetField(tif, EXIFTAG_EXPOSURETIME,raw_data))
		  im->exposure=((float *) raw_data)[0];
	      }
	  }
	
	TIFFClose(tif);
	return 1 ; 
    }
    return 0;
}

struct image * cropImage(struct image * source, int X,int Y, int l, int h)
{
  struct image * destination;
  destination = malloc(sizeof(struct image));
 
  destination->width = l;
  destination->height = h;
  destination->bufferSize = l*h*source->sampleParPixel*(source->bps/8);
  destination->sampleParPixel = source->sampleParPixel;
  destination->bps = source->bps;
  destination->exposure = source->exposure;
  
  if((destination->buffer = (char *) malloc(destination->bufferSize)) == NULL){
    fprintf(stderr, "Pas assez de memoire\n");
    exit(42);
  }
  /* printf("Crop %i %i %i %i %i\n", */
/* 	 X,Y,l,h,destination->bufferSize); */
/*   fflush(stdout); */
  
  int i,j,x,y;
  uint16 R,G,B;
  R=0;G=0;B=0;
  // Recherche de la fonction de recherche d'une valeur d'un pixel
  _ImVal ImVal  = DefImVal(source);
  _ImMaj ImMaj  = DefImMaj(destination);
  x=X;
 
  for (i=0;i<l;i++)
    {
      y=Y;
      for (j=0;j<h;j++)
	{
	  (ImVal)(source,x,y,&R,&G,&B);
	  (ImMaj)(destination,i,j,R,G,B);
	  y++;
	  
	}
      x++;
    }
 
  return destination;
}

void makeHistogramme(char* nTiff)
{
  struct image im ;
  int x,y,i;
  unsigned long count;
  loadTiff(nTiff, &im,A_TRUE);
   if (im.bufferSize>0)
    {
      // On divise par 255 pour simplifier
      int L[256];
      int R[256];
      int G[256];
      int B[256];

      
      for (i=0;i<255;i++) 
	{
	  L[i]=0;
	  R[i]=0;
	  G[i]=0;
	  B[i]=0;
	}

      for (y=0;y<im.height;y++)
	for (x=0;x<im.width;x++)
	  {
	    count = y*im.width+ x;
	    
	    R[((int)(im.buffer[count*6  ]+256*im.buffer[count*6+1])/256)]++;
	    G[((int)(im.buffer[count*6+2]+256*im.buffer[count*6+3])/256)]++;
	    B[((int)(im.buffer[count*6+4]+256*im.buffer[count*6+5])/256)]++;

	    L[(int)((im.buffer[count*6  ]+256*im.buffer[count*6+1]+
		     im.buffer[count*6+2]+256*im.buffer[count*6+3]+
		     im.buffer[count*6+4]+256*im.buffer[count*6+5])/(256*3))]++;

	  }
      free(im.buffer);
      int vMax=0;
      for (i=0;i<255;i++) vMax=max(L[i],vMax);
      int cut = vMax*0.03;
      int seuil_h = 0;
      for (i=255;i>0;i--) 
	if (L[i]<1000) seuil_h=i*256;
      
      for (i=0;i<255;i++) 
	{
	  printf("%i L%5i R%5i G%5i B%5i\n",i*256,L[i],R[i],G[i],B[i]);
	}
      printf("vMax %i cut %i seuil_b %i\n",vMax,cut,seuil_h);
    }
   
}

int analysePoint(char *txt, int *a,int *b)
{
  int i;
  int l = strlen(txt);
  int sep=-1;
  int err=0;
  for (i=0;i<l;i++)
    switch (txt[i])
      {
      case 'x':
      case 'X':
	sep=i;
	break;
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
	break;
      default:
	err=1;
	break;
      }
  if (err)
    return 0;
  *b=atoi(&txt[sep+1]);
  txt[sep]='\0';
  *a=atoi(txt);

  return 1;
}

void extractTiffGnuPlot(char* nTiff,
			char* AA,
			char* BB)
{

  /* extract a dump of a image with gnuplot syntaxe */
  struct image im ;
  loadTiff(nTiff, &im, A_TRUE);
  // Recherche de la fonction de recherche d'une valeur d'un pixel
  _ImVal ImVal  = DefImVal(&im);
  int x1,y1,x2,y2;
  analysePoint(AA,&x1,&y1);
  analysePoint(BB,&x2,&y2);
  int i,j;
  uint16 R,G,B;
  R=0;
  G=0;
  B=0;
  int cas;
  printf("x y R G B\n",i,j,R,G,B);
  for (i=x1;i<x2;i++)
    {
      for (j=y1;j<y2;j++)
	{
	  (ImVal)(&im,i,j,&R,&G,&B);
	  printf("%i %i %i %i %i \n",i,j,R,G,B);
	  
	}
      printf("\n");
    }
}

void statTiff(char* nTiff)
{
  struct image im ;
  loadTiff(nTiff, &im, A_TRUE);

  // Cette saturation ne sert que pour un canon 350D 
  int SATURATION = 4095;

  // Recherche de la fonction de recherche d'une valeur d'un pixel
  _ImVal ImVal  = DefImVal(&im);
  
  if (im.bufferSize>0)
    {
        int v;
	int mMax =0;
	int mMin = 32767;
	float  m = 0;
	float m2 = 0;
	float e =0;
	unsigned x,y;

	uint16 R,G,B;
	R=0;
	G=0;
	B=0;
	
	for (y=0;y<im.height;y++)
	  for (x=0;x<im.width;x++)
	    {
	      (ImVal)(&im,x,y,&R,&G,&B);
	      v=(R+G+B)/3;
	      m += v;
	      m2 += v*v;

	      if ((v > mMax)&&(v!=SATURATION))  mMax = v;
	      if (v < mMin)  mMin = v;	  
	    }

	m = m / (im.width*im.height*im.sampleParPixel);
	m2 = m2 / ((im.width*im.height*im.sampleParPixel)-1);
	e = sqrt(m2 - m*m);
	
	// RSB
	float RSB;
	unsigned nRSB;
	RSB=0;
	nRSB=0;
	int vMin;
	vMin = (int)(1.1*m);
	for (y=0;y<im.height;y++)
	  for (x=0;x<im.width;x++)
	    {
	      (ImVal)(&im,x,y,&R,&G,&B);
	      v=(R+G+B)/3;
	      if ((v>vMin)&&(v<mMax))
		{
		  RSB+=v;
		  nRSB++;
		}
	    }

	printf("Image %s\n",nTiff);
	printf("------------------\n");
	printf("Taille : %ix%i\n",im.height, im.width);
	printf("bit per sample = %i sample per plan =%i (3=RGB Image)\n",
	       im.bps, im.sampleParPixel);
	printf("Maximum = %d , Minimum = %d , moyenne = %f , ecart type =%f moyenne/ecart type = %f\n",
	       mMax , mMin , m ,e,m/e);
	if (nRSB!=0)
	  printf("RSB = %f %i\n",(float) RSB/(nRSB*m),vMin);
	printf("\n");

	
    }
}

void copyImage(struct image * source, struct image * destination)
{
  destination->width = source->width;
  destination->height = source->height;
  destination->bufferSize = source->bufferSize;
  destination->sampleParPixel = source->sampleParPixel;
  destination->bps = source->bps;
  destination->exposure = source->exposure;
  if((destination->buffer = (char *) malloc(destination->bufferSize)) == NULL){
    fprintf(stderr, "Pas assez de memoire\n");
    exit(42);
  }
  /* Attention la copie est que sur les donnees de structure pas le buffer */
}

int lireImageRep(char * nrep, struct image ** images)
{
  /*   Lire le repertoire des images */

  struct dirent *lecture;
  DIR *rep;
  rep = opendir(nrep);
  
  int i=0;

  /* lire les images */
  while ((lecture = readdir(rep))) {
    /* On prend que les fichiers .tiff */
    if  (strlen(lecture->d_name) > 5 )
	 
      {
	char *p = lecture->d_name;
	p = p + strlen(lecture->d_name)-5;
	if (strcmp(p, ".tiff")==0)
	  {
	    /*   Mettre en mÃ©moire les images */
	    images[i]=malloc(sizeof(struct image));
	    char nTiff[256];
	    sprintf(nTiff,"%s//%s",nrep,lecture->d_name);
	    printf("FICHIER: %s\n", nTiff);
	    fflush(stdout);
	    loadTiff(nTiff, images[i],A_TRUE);
	    i++;
	  }
      }
  }
  closedir(rep);
  return i;
}

struct image * doAdditionA(struct image * im,uint16 *R,uint16 *G,uint16 *B)
{
  unsigned long x,y;

  _ImVal ImVal  = DefImVal(im);

  // Image a generer
  struct image * gen;  
  gen=malloc(sizeof(struct image));
  
  gen->width = im->width;
  gen->height =  im->height;
  gen->sampleParPixel=3;
  gen->bps=16;
  gen->exposure=0;
  gen->bufferSize = gen->width * gen->height * gen->sampleParPixel *2;
  gen->buffer;
  if((gen->buffer = (char *) malloc(gen->bufferSize)) == NULL){
    fprintf(stderr, "Could not allocate enough memory for the uncompressed image\n");
    exit(42);
  }
  unsigned long count;
 
  uint16 Rv,Gv,Bv;
  Rv=0; Gv=0; Bv=0;
  // Initialisation avec la premiere image
  for (y=0;y<im->height;y++)
    for (x=0;x<im->width;x++)
      {
	(ImVal)(im,x,y,&Rv,&Gv,&Bv);
 
	count = y*im->width+ x;
	R[count]=Rv;//im->buffer[count*6]+256*im->buffer[count*6+1];
	G[count]=Gv;//im->buffer[count*6+2]+256*im->buffer[count*6+3];
	B[count]=Bv;//im->buffer[count*6+4]+256*im->buffer[count*6+5];
	/* if ((x==1640)&&(y==(2314-920))) */
/* 	  printf("1 : R %i G %i B %i\n",R[count],G[count],B[count]); */
      }
  gen->exposure=im->exposure;
  
  return gen;

}

void ventilation(float coefx,float coefy, uint16 v, 
	    uint16 *a, uint16 *b, uint16 *c, uint16 *d)
{
  float m1,m2,m3;
  int32 M1,M2,M3;
  
  m1 = (1.0-coefx)*(1.0-coefy)*v;
  m2 = coefx*(1.0-coefy)*v;
  m3 = (1.0-coefx)*coefy*v;

/*   m1 = coefx*coefy*v; */
/*   m2 = (1.0-coefx)*coefy*v; */
/*   m3 = coefx*(1-coefy)*v; */

  M1 = m1<0.0?0:m1>65535.0?65535:m1;
  M2 = m2<0.0?0:m2>65535.0?65535:m2;
  M3 = m3<0.0?0:m3>65535.0?65535:m3;


  // On arrondi a l'entier superieur
 /*  if ((m1-M1)>0.5) M1++; */
/*   if ((m2-M2)>0.5) M2++; */
/*   if ((m3-M3)>0.5) M3++; */

  *a=(uint16)M1;
  *b=(uint16)M2;
  *c=(uint16)M3;
  // On perds pas de valeur de pixel en route ...
  *d=(uint16)((int32)v-M1-M2-M3);

/*   if ((*a+*b+*c+*d)!=v) */
/*     printf("Erreur %i %i %i %i = %i \n", *a, *b,*c,*d,v); */


 
}

int doAdditionB(struct image * im, float * trans,uint16 *R,uint16 *G,uint16 *B,struct image * gen)
{
  unsigned long x,y, count;

  float xp,yp,coefx,coefy;
  uint16 Rv,Gv,Bv;
  unsigned long xi,yi;

  _ImVal ImVal  = DefImVal(im);

  // Addition
  for (y=0;y<im->height;y++)
    for (x=0;x<im->width;x++)
      {
	// trans contient les valeurs ABC DEF
	xp=trans[0]+trans[1]*x+trans[2]*y;
	yp=trans[3]+trans[4]*x+trans[5]*y;
	

	// On est dans le cadre ?
	if ((xp>0)&&(xp<(im->width-1))&&(yp>0)&&(yp<(im->height-1)))
	  {

	    (ImVal)(im,x,y,&Rv,&Gv,&Bv);

/* 	    count = y*im->width+ x; */
/* 	    Rv=im->buffer[count*6]+256*im->buffer[count*6+1]; */
/* 	    Gv=im->buffer[count*6+2]+256*im->buffer[count*6+3]; */
/* 	    Bv=im->buffer[count*6+4]+256*im->buffer[count*6+5]; */
	    
	    xi=xp;
	    yi=yp;

	    // On ventile sur les quatres pixels voisins
	    
	    //coefx=1.0-(xp-xi);
	    //coefy=1.0-(yp-yi);
	    coefx=xp-xi;
	    coefy=yp-yi;

	    uint16 Ra, Rb, Rc, Rd;
	    uint16 Ga, Gb, Gc, Gd;
	    uint16 Ba, Bb, Bc, Bd;
	    	    
	    ventilation(coefx,coefy, Rv,&Ra,&Rb,&Rc,&Rd); 
	    ventilation(coefx,coefy, Gv,&Ga,&Gb,&Gc,&Gd); 
	    ventilation(coefx,coefy, Bv,&Ba,&Bb,&Bc,&Bd); 

	    // Attention au saturation !

	    count = yi*im->width+ xi;
	    R[count]=R[count]>(65534-Ra)?65535:R[count]+Ra;
	    G[count]=G[count]>(65534-Ga)?65535:G[count]+Ga;
	    B[count]=B[count]>(65534-Ba)?65535:B[count]+Ba;

	    count = yi*im->width+ xi+1;
	    R[count]=R[count]>(65534-Rb)?65535:R[count]+Rb;
	    G[count]=G[count]>(65534-Gb)?65535:G[count]+Gb;
	    B[count]=B[count]>(65534-Bb)?65535:B[count]+Bb;
	    
	    count = (yi+1)*im->width+ xi;
	    R[count]=R[count]>(65534-Rc)?65535:R[count]+Rc;
	    G[count]=G[count]>(65534-Gc)?65535:G[count]+Gc;
	    B[count]=B[count]>(65534-Bc)?65535:B[count]+Bc;

	    count = (yi+1)*im->width+ xi+1;
	    R[count]=R[count]>(65534-Rd)?65535:R[count]+Rd;
	    G[count]=G[count]>(65534-Gd)?65535:G[count]+Gd;
	    B[count]=B[count]>(65534-Bd)?65535:B[count]+Bd;
   
	    
	  }
      }
  gen->exposure+=im->exposure;
}
int doAdditionC(char *nomImage, uint16 *R,uint16 *G,uint16 *B,struct image * gen,
		int nbImage)
{
  unsigned long x,y,count;

  for (y=0;y<gen->height;y++)
    for (x=0;x<gen->width;x++)
      {

	count = y*gen->width+ x;
	// Coorection balance 
	R[count]*=BALANCE_R;
	G[count]*=BALANCE_G;
	B[count]*=BALANCE_B;

	/* // Moyenne des valeurs */
	if (nbImage !=0) {
	  R[count]/=nbImage;
	  G[count]/=nbImage;
	  B[count]/=nbImage;
	}
	
	
/* 	if ((x==1640)&&(y==(2314-920))) */
/* 	  printf("3 R %i G %i B %i\n",R[count],G[count],B[count]); */
/* 	if (R[count]>65535) { */
/* 	  gen->buffer[count*6+1]=255; */
/* 	  gen->buffer[count*6]  =255; */
/* 	} */
/* 	else { */
	  gen->buffer[count*6+1]=R[count]/256;
	  gen->buffer[count*6]  =R[count]-gen->buffer[count*6+1]*256;
/* 	} */
/* 	if (G[count]>65535) { */
/* 	  gen->buffer[count*6+3]=255; */
/* 	  gen->buffer[count*6+2]=255; */
/* 	} */
/* 	else { */
	  gen->buffer[count*6+3]=G[count]/256;
	  gen->buffer[count*6+2]=G[count]-gen->buffer[count*6+3]*256;
/* 	} */
/* 	if (B[count]>65535) { */
/* 	  gen->buffer[count*6+5]=255; */
/* 	  gen->buffer[count*6+4]=255; */
/* 	} */
/* 	else { */
	  gen->buffer[count*6+5]=B[count]/256;
	  gen->buffer[count*6+4]=B[count]-gen->buffer[count*6+5]*256;
/* 	} */
      }
  // fichier fic dynamique a faire
  char fic[500]; // buffer dynamique a faire
  sprintf(fic,"%s//%s.tiff","results",nomImage);
  writeTiff(gen, fic);
  printf("Exposition total :%.3f nbImage %i\n",gen->exposure,nbImage);
  // Nettoyage de la mémoire
  free(R);
  free(G);
  free(B);
  free(gen->buffer);
}

struct image * doZoomAdditionA(char *nomImageA,uint16 *R,uint16 *G,uint16 *B,
			       int zX,int zY,int zL, int zH, int zZ,
			       float *tP, float *tPmax)
{
  unsigned long x,y,z1,z2;
  printf("Image servant de base %s :\n",nomImageA);
  fflush(stdout);
  
  //Lecture fichier source
  struct image im;
  char fic[500]; // buffer dynamique a faire
  sprintf(fic,"%s//%s.tiff","results",nomImageA);
  loadTiff(fic, &im,A_TRUE);


// Image Ã  gÃ©nÃ©rer
  struct image * gen;  
  gen=malloc(sizeof(struct image));
  
  gen->width = zL*zZ;
  gen->height =  zH*zZ;
  gen->sampleParPixel=3;
  gen->bps=16;
  gen->exposure=0;
  gen->bufferSize = gen->width * gen->height * gen->sampleParPixel *2;
  gen->buffer;
  if((gen->buffer = (char *) malloc(gen->bufferSize)) == NULL){
    fprintf(stderr, "Could not allocate enough memory for the uncompressed image\n");
    exit(42);
  }
  unsigned long count,countZ;
  tPmax[0]=1.0f;
  for (count=0;count<zL*zH*zZ*zZ;count++)
    {
      R[count]=1;
      G[count]=1;
      B[count]=1;
      tP[count]=0.0f;
      
    }
  
  // Initialisation avec la premiere image
  tPmax[0]=1.0f; // En ideal c'estle temps d'exposition im.exposure;
  for (y=0;y<zH;y++)
    for (x=0;x<zL;x++)
      for (z1=0;z1<zZ;z1++)
	for (z2=0;z2<zZ;z2++)
	
	{
	  count = (zY+y)*im.width+ x+zX;
	  countZ = (y+z2)*zL*zZ+x*zZ+z1;
	  R[countZ]=im.buffer[count*6]+256*im.buffer[count*6+1];
	  G[countZ]=im.buffer[count*6+2]+256*im.buffer[count*6+3];
	  B[countZ]=im.buffer[count*6+4]+256*im.buffer[count*6+5];
	  tP[countZ]=1.0f; // En ideal c'est le temsp d'exposition im.exposure;
	  /* 	if ((x==1640)&&(y==(2314-920)))  */
	  /* 	  printf("R %i G %i B %i\n",R[count],G[count],B[count]); */
	}
  gen->exposure=im.exposure;
  free(im.buffer);
  
  return gen;

}

int doZoomAdditionB(char *nomImageB, float * trans, uint16 *R, uint16 *G,uint16 *B,
		    struct image * gen,
		    int zX,int zY,int zL, int zH, int zZ,
		    float *tP, float *tPmax)
{
  unsigned long x,y,count,countZ,z1,z2;
  // Addition image 
  struct image im;
  char fic[500]; // buffer dynamique a faire
  sprintf(fic,"%s//%s.tiff","results",nomImageB);
  loadTiff(fic, &im,A_TRUE);
  float xp,yp,coefx,coefy,xpz,ypz;
  uint16 Rv,Gv,Bv;
  int xi,yi;
  // Addition
  for (y=0;y<im.height;y++)
    for (x=0;x<im.width;x++)
      {
	// trans contient les valeurs ABC DEF
	xp=trans[0]+trans[1]*x+trans[2]*y;
	yp=trans[3]+trans[4]*x+trans[5]*y;
	/* if ((x==1640)&&(y==(2314-920))) */
/* 	  printf("   %f %f count %i\n",xp,yp,y*im.width+ x); */

	// On est dans le cadre ?
	if ((xp>zX)&&(xp<(zX+zL))&&(yp>zY)&&(yp<(zY+zH)))
	  {
	    count = y*im.width+ x;
	    
	    /* 	    Rv=im.buffer[count*6]+256*im.buffer[count*6+1]; */
	    /* 	    Gv=im.buffer[count*6+2]+256*im.buffer[count*6+3]; */
	    /* 	    Bv=im.buffer[count*6+4]+256*im.buffer[count*6+5]; */
	    
	    for (z1=0;z1<zZ;z1++)
	      for (z2=0;z2<zZ;z2++)
		{
		  xpz=(xp-zX)*zZ+z1; 
		  ypz=(yp-zY)*zZ+z2; 
		  countZ = ((int)ypz)*zL*zZ+(int)xpz;
		  //countZ = ((int) yp -zY)*zL*zZ+((int) xp-zX)*zZ;
		  if ((countZ>=0) &&(countZ<(zL*zH*zZ*zZ)))
		    {
		      R[countZ]+=im.buffer[count*6]+256*im.buffer[count*6+1];
		      G[countZ]+=im.buffer[count*6+2]+256*im.buffer[count*6+3];
		      B[countZ]+=im.buffer[count*6+4]+256*im.buffer[count*6+5];
		      tP[countZ]+=1.0f; // ideal c'est le temps d'exposition im.exposure;
		      if (tP[countZ] > tPmax[0] ) *tPmax = tP[countZ];
		    }
		}
	    
	  }
      }
  gen->exposure+=im.exposure;
  free(im.buffer);
}

int doZoomAdditionC(char *nomImage, uint16 *R,uint16 *G,uint16 *B,struct image * gen,
		    float *tP, float tPmax, int iT)
{
  uint16 x,y;
  uint32 count;

  for (y=0;y<gen->height;y++)
    for (x=0;x<gen->width;x++)
      {
	count = y*gen->width+ x;
	// Normalisation par rapport au maximum d'exposition
	if (tP[count]!=0) {
	  R[count]=R[count]*tPmax/tP[count];
	  G[count]=G[count]*tPmax/tP[count];
	  B[count]=B[count]*tPmax/tP[count];
	}
      }

  doAdditionC(nomImage,R,G,B,gen,iT);
}

void doTrt(char *nomImage,struct image * gen , struct image *offset,struct image * dark, struct image * flat)
{

  uint16 *R;
  uint16 *G;
  uint16 *B;
  unsigned long count;
  unsigned long  tImage = gen->width*gen->height;

  unsigned long x,y;
  int cas;

  R = malloc(sizeof(uint16)*tImage);
  G = malloc(sizeof(uint16)*tImage);
  B = malloc(sizeof(uint16)*tImage);

  //Lecture fichier source
  struct image im;
  char fic[500]; // buffer dynamique a faire
  sprintf(fic,"%s//%s.tiff","lights",nomImage);
  loadTiff(fic, &im,A_TRUE);


  // Initialisation des couleurs + calcul flat moyen par couleur
  
  
  float flatMoyR=0;
  float flatMoyG=0;
  float flatMoyB=0;

  for (y=0;y<im.height;y++)
    {

      for (x=0;x<im.width;x++)
	{
	  count = y*im.width+ x;

	  R[count]=0;
	  G[count]=0;
	  B[count]=0;
	  cas=x%2 + (y % 2)*10;
	  // si x pair et y pair 0
	  // si x pair et y impair 10
	  // si x impair et y pair 1
	  // si x impair et y impair 11
	  if (cas ==0)               flatMoyR += flat->buffer[count*2]+256*flat->buffer[count*2+1];
	  if ((cas ==10)||(cas ==1)) flatMoyG += flat->buffer[count*2]+256*flat->buffer[count*2+1];
	  if (cas ==11)              flatMoyB += flat->buffer[count*2]+256*flat->buffer[count*2+1];

/* 	  if (cas ==0)               flatMoyR=max(flatMoyR,flat->buffer[count*2]+256*flat->buffer[count*2+1]); */
/* 	  if ((cas ==10)||(cas ==1)) flatMoyG=max(flatMoyG,flat->buffer[count*2]+256*flat->buffer[count*2+1]); */
/* 	  if (cas ==11)              flatMoyB=max(flatMoyB,flat->buffer[count*2]+256*flat->buffer[count*2+1]); */

	}
    }
  flatMoyR =  flatMoyR / (tImage/4);
  flatMoyG =  flatMoyG / (tImage/2);
  flatMoyB =  flatMoyB / (tImage/4);

  //printf(" flat Moyen : %f %f %f\n",flatMoyR,flatMoyG,flatMoyB);
    
  int32 v,off,dar,fla;
  
  int mMax=0;
  unsigned long nbZero=0;
  float  moyenne = 0;
  float m2 = 0;
  float ecart_type =0;
  struct etoile **etoiles;
  etoiles = malloc(sizeof(struct etoile *)*im.height);

  _ImVal ImValD = DefImVal(dark);
  _ImVal ImValF = DefImVal(flat);
  _ImVal ImValO = DefImVal(offset);
  _ImVal ImVal  = DefImVal(&im);

  uint16 value,none;
  value=0;
  none=0;

  uint16 mR,MR;
  uint16 mG,MG;
  uint16 mB,MB;
  mR=65535;  mG=65535;  mB=65535;
  MR=0;MG=0;MB=0;
       
  // Decodage matrice de bayer 
  for (y=0;y<im.height;y++)
    {
      etoiles[y]=NULL;
      for (x=0;x<im.width;x++)
	{
	  cas=x%2 + (y % 2)*10;
	  // si x pair et y pair 0
	  // si x pair et y impair 10
	  // si x impair et y pair 1
	  // si x impair et y impair 11

	  count = y*im.width+ x;
	  (ImVal)(&im,x,y,&value,&none,&none);
	  v = value ; //im.buffer[count*2]+256*im.buffer[count*2+1];

	  //offset et dark
	  (ImValO)(offset,x,y,&value,&none,&none);
	  off = value;
	  //off = offset->buffer[count*2]+256*offset->buffer[count*2+1];
	  //dar = dark->buffer[count*2]+256*dark->buffer[count*2+1];
	  (ImValD)(dark,x,y,&value,&none,&none);
	  dar = value;
	  
	  //fla = flat->buffer[count*2]+256*flat->buffer[count*2+1];
	  (ImValD)(flat,x,y,&value,&none,&none);
	  fla = value;
	  // On applique le flat avant le dark ?
	  
	  float flatMoy;
	  
	  if (cas ==0)               flatMoy = flatMoyR;
	  if ((cas ==10)||(cas ==1)) flatMoy = flatMoyG;
	  if (cas ==11)              flatMoy = flatMoyB;

	  if (cas ==0){               mR = min(mR,fla); MR=max(MR,fla);}
	  if ((cas ==10)||(cas ==1)){ mG = min(mG,fla); MG=max(MG,fla);}
	  if (cas ==11){              mB = min(mB,fla); MB=max(MB,fla);}

	  float calcul;
	  if (fla!=0)
	    calcul =  COEF_FLAT *(v+SEUIL-off-dar)*flatMoy/fla;
	  else
	    calcul = 0;
	  v = calcul > 65535 ? 65535 : calcul < 0 ? 0 : (uint16) calcul;
	  //v = COEF_FLAT*(v+SEUIL-off-dar)*flatMoy/fla;
	  //v =  COEF_FLAT*(v-off-dar+SEUIL);

	  //v = v>65535?65535:v;

	  if (v<0) 
	    {
	      v=0;
	      nbZero++;
	    }

	  if (cas ==0) 
	    {
	      R[count]=(uint16)v;
	      
	    }

	  if ((cas ==10)||(cas ==1))
	    {
	      G[count]=(uint16)v;
	      // On calcul la moyenne au passage pour le vert
	      moyenne += v;
	      m2+= v*v;
	      mMax=max(mMax,v);
	    }
	  if (cas ==11) B[count]=(uint16)v;

	}
    }
  moyenne = moyenne / (im.width*im.height/2);
  m2 = m2 / (im.width*im.height/2);
  ecart_type = sqrt(m2 - moyenne*moyenne);
/*   printf("Nb de valeur inferieur a zero apres pre-traitement : %i soit %f\n ",  */
/* 	 nbZero, */
/* 	 nbZero*100/(im.width*im.height)); */

/*   printf(" min flat Moyen : %i %i %i\n",mR,mG,mB); */
/*   printf(" max flat Moyen : %i %i %i\n",MR,MG,MB); */
	 
  //printf("Stat sur le rouge : m=%f %f ecart=%f\n", moyenne,m2,ecart_type);
  //exit(0);

  int32 vR, vG, vB;
  int nR;
  int nG;
  int nB;
  struct etoile *e;
  struct point *Point;

  // Valeur de coupure pour la détection des étoiles
  uint16 vCoupure;
  vCoupure=moyenne+2*ecart_type;
  //vCoupure=mMax*50/100;
  //printf("mMax %i %i %i %i \n",mMax,vCoupure,(int)ecart_type,(int)moyenne );

  // Interpolation et preparation buffer
  for (y=0;y<im.height;y++) {
    
    /* Recherche des etoiles */
   int etat = 1;
    //int debug = 1;

    for (x=0;x<im.width;x++)
      {

	cas=x%2 + (y % 2)*10;
	count = y*im.width+ x;
	// si x pair et y pair 0
	// si x pair et y impair 10
	// si x impair et y pair 1
	// si x impair et y impair 11

	if (cas==0)
	  {
	    // Calcul du vert
	    nG=0;
	    vG=0;
	    if (x>0)             { vG += G[y*im.width+ x-1]; nG++;}
	    if (x<(im.width-1))  { vG += G[y*im.width+ x+1]; nG++;}
	    if (y>0)             { vG += G[(y-1)*im.width+ x]; nG++;}
	    if (y<(im.height-1)) { vG += G[(y+1)*im.width+x]; nG++;}
	    G[count]=(uint16)vG/nG;
	    // Calcul du bleu
	    nB=0;
	    vB=0;
	    
	    if ((x>0) &&( y>0)) {  vB += B[(y-1)*im.width+x-1]; nB++;}
	    if ((x<(im.width-1))&& (y>0))  { vB += B[(y-1)*im.width+ x+1]; nB++;}
	    if ((x>0)  && ( y<(im.height-1))) { vB += B[(y+1)*im.width+ x-1]; nB++;}
	    if ((x<(im.width-1))&& (y<(im.height-1)))  { vB += B[(y+1)*im.width+ x+1]; nB++;}
	    B[count]=(uint16)vB/nB;
	  }

	if (cas==1)
	  {
	    // Calcul du rouge
	    nR=0;
	    vR=0;
	    if (x>0)            { vR += R[y*im.width+ x-1]; nR++;}
	    if (x<(im.width-1)) { vR += R[y*im.width+ x+1]; nR++;}
	    R[count]=(uint16)vR/nR;
	    // Calcul du bleu
	    nB=0;
	    vB=0;
	    if (y>0)              { vB += B[(y-1)*im.width+ x]; nB++;}
	    if (y<(im.height-1))  { vB += B[(y+1)*im.width+ x]; nB++;}
	    B[count]=(uint16)vB/nB;
	  }
	if (cas==10)
	  {
	    // Calcul du bleu
	    nB=0;
	    vB=0;
	    if (x>0)                { vB += B[y*im.width+ x-1]; nB++;}
	    if (x<(im.width-1))     { vB += B[y*im.width+ x+1]; nB++;}
	    B[count]=(uint16)vB/nB;
	    // Calcul du rouge
	    nR=0;
	    vR=0;
	    if (y>0)              { vR += R[(y-1)*im.width+ x]; nR++;}
	    if (y<(im.height-1))  { vR += R[(y+1)*im.width+ x]; nR++;}
	    R[count]=(uint16)vR/nR;
	  }
	if (cas==11)
	  {
	    // Calcul du rouge
	    nR=0;
	    vR=0;

	    if ((x>0)&&(y>0))             { vR += R[(y-1)*im.width+ x-1]; nR++;}
	    if ((x>0)&&(y<(im.height-1))) { vR += R[(y+1)*im.width+ x-1]; nR++;}
	    if ((x<(im.width-1))&&(y>0))             { vR += R[(y-1)*im.width+ x+1]; nR++;}
	    if ((x<(im.width-1)) &&(y<(im.height-1))) { vR += R[(y+1)*im.width+ x+1]; nR++;}
	    R[count]=(uint16)vR/nR;
	    // Calcul du vert
	    nG=0;
	    vG=0;
	    if (x>0)                 { vG += G[y*im.width+ x-1]; nG++;}
	    if (x<(im.width-1))      { vG += G[y*im.width+ x+1]; nG++;}
	    if (y>0)                 { vG += G[(y-1)*im.width+ x]; nG++;}
	    if (y<(im.height-1))     { vG += G[(y+1)*im.width+x]; nG++;}
	    G[count]=(uint16)vG/nG;

	    }
	vR=R[count];
	vG=G[count];
	vB=B[count];
	gen->buffer[count*6+1]=vR/256;
	gen->buffer[count*6]  =vR-gen->buffer[count*6+1]*256;
	gen->buffer[count*6+3]=vG/256;
	gen->buffer[count*6+2]=vG-gen->buffer[count*6+3]*256;
	gen->buffer[count*6+5]=vB/256;
	gen->buffer[count*6+4]=vB-gen->buffer[count*6+5]*256;

/*     if ((y==1979)&&(x==826)){ */
/*       printf("Ici %ix%i = %i,%i,%i cas : %i\n",x,y,vR,vG,vB,cas); */
/*       fflush(stdout); */
/*     } */
	
	/* Recherche des Ã©toiles */

	// On traite la ligne des verts ?
	
	uint16 p = G[count];
        //if (p < (moyenne+2*ecart_type)) p = 0 ;
	if (p < vCoupure) p = 0 ;
	/* 	debug=0; */
/* 	if ((y >= 105) && (y < 115) &&(x >= 2895) && (x < 2905) ) */
/* 	  { */
/* 	    debug=1; */
/* 	    printf ("%ix%i %i Etat %i\n",x,y,p,etat); */
/* 	    // printf ("debut %i%i fin %i %i \n",e.debut.x,e.debut.y,e.fin.x,e.fin.y); */
/* 	  } */
	
        if ((p != 0) && (etat==1))
	  {
	    // début d'une étoile
            etat = 0;
	    
            struct point pp;
	    e = malloc(sizeof(struct etoile));
            pp.x=x;
            pp.y=y;
            e->debut=pp;
	    e->fin.x=0;
	    e->fin.y=0;
	    e->intensite=0;
	    e->points=NULL;
	    e->suivant=NULL;
	    e->indice = -1;
	    e->vMax=0;
	    //if (debug) printf("Debut x %i,y%i \n",e->debut.x,e->debut.y);
	  }
        if ((p == 0) && (etat==0))
          {
	    // Fin d'une etoile
            struct point pp;
            pp.x=x;
            pp.y=y;
            e->fin=pp;
	    //if (debug) printf(" fin x %i,y%i \n",e->fin.x,e->fin.y);
            // On cherche sur la ligne prÃ©cÃ©dente une Ã©toile
            if ((y > 0) && (etoiles[y-1]!=NULL))
	      {
		struct etoile *ee=etoiles[y-1];
		struct etoile *pee=NULL;
		
		int ok=0;
		do
		  {
		    //struct etoile * ee =pe;
		    if ((min(ee->fin.x,e->fin.x) - max(ee->debut.x,e->debut.x))>0)
		      {
			// ok
			ok=1;

                        e->debut.x= min(ee->debut.x,e->debut.x);
                        e->fin.x= max(ee->fin.x,e->fin.x);
                        e->debut.y=ee->debut.y;

                        e->intensite+=ee->intensite;
                        e->indice=ee->indice;

			e->vMax=max(e->vMax,ee->vMax);
			// Copie des points
			if (e->points==NULL)
			  e->points=ee->points;
			else
			  {
			    struct point *pse=e->points;
			    while (pse->suivant!=NULL) pse=pse->suivant; 
			    pse->suivant = ee->points;
			  }
			// On supprime l'ancienne ligne
			if (pee==NULL)
			  etoiles[y-1]=ee->suivant;
			else
			  pee->suivant=ee->suivant;
			/* if (etoiles[y-1]==ee)  */
			/* 			  { */
			/* 			    etoiles[y-1]=NULL; */
			/* 			    ee=ee->suivant; */
			/* 			  } */
		      }
			
		    pee=ee;
		    ee=ee->suivant;
		    
		  } while ((ee!=NULL)&&(ok==0));
	      }
	    // Stockage des informations
	    if (etoiles[y]!=NULL)
	      {
		struct etoile *pe=etoiles[y];
		while (pe->suivant!=NULL) pe=pe->suivant;
		pe->suivant=e;
	      }
            else
	      {
                etoiles[y]=e;
	      }
            
            etat = 1;
	  }
	
	if (etat==0)
	  {
	    // Etoile en cours
	    e->intensite+=p;
	    e->vMax=max(e->vMax,p);
 
	    Point = malloc(sizeof(struct point));
	    Point->x=x;
	    Point->y=y;
	    Point->intensite=p;
	    Point->suivant=NULL;
	    
	    if (e->points==NULL)
	      e->points=Point;
	    else
	      {
		struct point *ppp=e->points;
		while (ppp->suivant!=NULL)  ppp=ppp->suivant;
		ppp->suivant=Point;
		
		ppp=e->points;
		while (ppp!=NULL) ppp=ppp->suivant;
	      }
	    
	  }
	
	
	/* debug */
	
/* 	if ((x<10) && (y<(10))) */
/* 	  { */
/* 	    printf ("x %i y %i v %i R %i G %i B %i \n",x,y,v,vR,vG,vB); */
/* 	  } */
      }
  }
  
  // Calcul sur étoiles
  // Qualite de l'image
  // Affichage des etoiles
  // Ecriture sur disk des étoiles
  sprintf(fic,"%s//%s.data","results",nomImage);

  float intensiteMoyenne=0;
  float FWHMMoyen=0;
  int nbetoiles=0;

  FILE *pf = fopen(fic,"w");

  for (y=0;y<im.height;y++) 
    {

      if (etoiles[y]!=NULL)
	
	{
	  struct etoile *pe=etoiles[y];
	  do
	    {
	      // Calcul barycentre
	      float  xM = 0;
	      float  yM = 0;
	      float  M = 0;
	      struct point *ppp=pe->points;
	      while (ppp!=NULL)
		{
		  xM +=ppp->x*ppp->intensite;
		  yM +=ppp->y*ppp->intensite;
		  M +=ppp->intensite;
		  //printf(" p (%i,y%i) %i \n",ppp->x,ppp->y,ppp->intensite); ;
		  ppp=ppp->suivant;
		}
	      pe->centre_X=xM/M;
	      pe->centre_Y=yM/M;
	      
	      // Calcul FWHM  420,809,423,811,intensite 2818,vMax 1385,FWHM 0.000000,421.00,810.00
	      // On prend le point superieur au FWMH le plus loin ...
	      int iFWHM = pe->vMax/2;
	      float d;
	      float dMax=0;
	      ppp=pe->points;
	      while (ppp!=NULL)
		{
		  if (ppp->intensite > iFWHM)
		    {
		      d = ((float)ppp->x - (float)pe->centre_X)*
			((float)ppp->x - (float)pe->centre_X)+
			((float)ppp->y - (float)pe->centre_Y)*
			((float)ppp->y - (float)pe->centre_Y);
		      dMax=max(d,dMax);
		    }
		  ppp=ppp->suivant;	
		}
	      pe->FWHM=sqrt(d)*2;
	      
	      //printf ("Debut %ix%i, fin %ix%i, intensite %i, vMax %i FWHM %.2f   Centre %.2fx%.2f\n",

/* 	      int surface=(pe->fin.x-pe->debut.x)*(pe->fin.y-pe->debut.y); */
/* 	      if (surface!=0) */
/* 		pe->intensite/=surface; */
/* 	      else */
/* 		pe->intensite=0; */

	      // On filtre que les trucs qui ont FWHM dans certaines bornes


	      if ((pe->FWHM>=0.0)&&
		  (pe->FWHM<100)&&
		  //(pe->intensite>moyenne+3*ecart_type)&&
		  //(pe->vMax<(mMax-5))&& 
		  ((pe->fin.y-pe->debut.y)>1)&&
		  ((pe->fin.x-pe->debut.x)>1))
		{
		  fprintf (pf,
			   "%i,%i,%i,%i,%i,%i,%.2f,%.2f,%.2f\n",
			   pe->debut.x,
			   pe->debut.y,
			   pe->fin.x,
			   pe->fin.y,
			   pe->intensite,
			   pe->vMax,
			   pe->FWHM,
			   pe->centre_X,pe->centre_Y);
		  int surface=(pe->fin.x-pe->debut.x)*(pe->fin.y-pe->debut.y);
		  intensiteMoyenne+= pe->intensite/surface;
		  FWHMMoyen+=pe->FWHM;
		  nbetoiles++;
		  
		}
	      pe=pe->suivant;	    
	    }
	  while (pe!=NULL);
	    
	 }
    }
  printf ("Qualite de l'image :  intensite moyenne %.3f FWHM moyen %.3f\n",
	  intensiteMoyenne/nbetoiles,
	  FWHMMoyen/nbetoiles);
  fclose(pf);

  // Nettoyage de la mémoire
  free(e);
  free(Point);
  free(R);
  free(G);
  free(B);
  free(etoiles);
  free(im.buffer);

}

int cmp_etoile(const void *x, const void  *y){
  struct etoile **A=(struct etoile **)x;
  struct etoile **B=(struct etoile **)y;
  
  uint32 a=  (*A)->intensite;
  uint32 b=  (*B)->intensite;

  //uint16 a=  (*A)->vMax;
  //uint16 b=  (*B)->vMax;

  if (a<b) return 1;
  if (a>b) return -1;
  return 0;
}

struct etoile * lectureFichier(char *nFic)
{
  printf("========\n%s\n=========\n",nFic);
  fflush(stdout);
  char fic[500]; //buffer dynamique a faire
  sprintf(fic,"%s//%s.data","results",nFic);

  struct etoile *pe;
  struct etoile *ppe=NULL;
  struct etoile *etoiles=NULL;

  FILE *pf = fopen(fic,"r");
  char buffer[512];// buffer dynamique 
  //L'ouverture du fichier est-elle realisee ? 
  if (pf != NULL)
    {
      while (fgets (buffer, sizeof buffer, pf) != NULL)
	{
	  pe = malloc(sizeof(struct etoile));
	  sscanf(buffer,"%i,%i,%i,%i,%i,%i,%f,%f,%f%*",
		 &pe->debut.x,
		 &pe->debut.y,
		 &pe->fin.x,
		 &pe->fin.y,
		 &pe->intensite,
		 &pe->vMax,
		 &pe->FWHM,
		 &pe->centre_X,&pe->centre_Y
		 );
	  pe->indice=-1;
	  pe->suivant=NULL;

	  // triage en plus 

	  if ((pe->FWHM>0.0))
	    {
	      if (etoiles==NULL) 
		{
		  etoiles = pe;
		  ppe=pe;
		}
	      else
		{
		  ppe->suivant=pe;
		}
	      ppe=pe;
	      


	    }
	}
      
      
    }
  fclose(pf);

  return etoiles;
}



struct triangle * creationTriangles(struct etoile * etoiles)
{
  struct triangle * triangles;
  struct triangle * pt;
  struct triangle * ppt;
  struct etoile *ppe=NULL;
  ppe=etoiles;
  int l=0;
  while (ppe!=NULL)
    {
      l++;
      ppe=ppe->suivant;
    }
  printf("nb d'etoiles = %i\n",l);
  struct etoile **tab;
  tab=malloc(sizeof(struct etoile*)*l);
  
  int i;
  ppe=etoiles;
  for (i=0;i<l;i++)
    {      
      tab[i]=ppe;
      ppe=ppe->suivant;
    }
  
  qsort( tab,
  	 l, /* le nombre d'elemnts */
	 sizeof(struct etoile *), /* la taille d'un element */
	 cmp_etoile);    /* la fonction qui compare les elements */

  // On prend les NBOBJ plus intenses
  int nbObj = NBOBJ;
  if (l>nbObj) l=nbObj;
  else nbObj=l;
  for (i=0;i<l;i++)
    {
      tab[i]->indice=i;
    }
  // Les variables
  int ia,ib,ic;
  float xt,yt;
  // Distance entre deux points
  float d[NBOBJ][NBOBJ]; // tableau dynamique a faire
  //d=malloc(sizeof(float)*nbObj*nbObj);
  for (ia=0;ia<nbObj;ia++)
    for (ib=0;ib<nbObj;ib++)
      {
	d[ia][ib]=sqrt( pow(tab[ib]->centre_X-tab[ia]->centre_X,2)+
			pow(tab[ib]->centre_Y-tab[ia]->centre_Y,2));
      }
  // Creaation des triangles
  triangles=NULL;
  float a,b,c,t;
  int nbe=0;
  for (ia=0;ia<nbObj;ia++)
    for (ib=ia+1;ib<nbObj;ib++)
      for (ic=ib+1;ic<nbObj;ic++)
	{
	  a=d[ia][ib];
	  b=d[ib][ic];
	  c=d[ia][ic];
	  
	  // Classement a>b>c
	  if (c>b) {t=c;c=b;b=t;} 
	  if (b>a) {t=a;a=b;b=t;} 
	  if (c>b) {t=c;c=b;b=t;}
	  //printf("a %f b %f c %f\n",a,b,c);
	  
	  xt=b/a;
	  yt=c/a;
	  if (xt<0.9)
	    {
	      pt=malloc(sizeof(struct triangle));
	      pt->iA=ia;
	      pt->iB=ib;
	      pt->iC=ic;
	      pt->xt=xt;
	      pt->yt=yt;
	      pt->suivant=NULL;
	      if (triangles==NULL) triangles=pt;
	      else ppt->suivant=pt;
	      ppt=pt;
	      nbe++;
	    }
	}
  
  printf("nb triangles crees = %i\n",nbe);
  return triangles;
}


float coefTranslation(float *cX,float *cZ)
//float A,float B,float C)
{

  // a +x =z
  int i;
  float X,Z;
  X=0;
  Z=0;
  // Calcul
  for (i=0;i<6;i++)
    {
      Z+=cZ[i];
      X+=cX[i];
    }
  return (Z-X);
}



int coefTransformation(float *cX,float *cY,float *cZ,float *s)
//float A,float B,float C)
{
  // Calcul des coeeficients methode moindre carre
  float e[4][3];
  //float s[3];
  int i,j;
  // Initialisation
  for (i=0;i<4;i++) 
    for (j=0;j<3;j++)
      e[i][j]=0.0;
  // Calcul
  for (i=0;i<6;i++)
    {
      // Equation 1
      e[0][0]+=1;
      e[1][0]+=cX[i];
      e[2][0]+=cY[i];
      e[3][0]+=cZ[i];
      // Equation 2
      e[0][1]+=cX[i];
      e[1][1]+=cX[i]*cX[i];
      e[2][1]+=cX[i]*cY[i];
      e[3][1]+=cX[i]*cZ[i];
      // Equation 3
      e[0][2]+=cY[i];
      e[1][2]+=cX[i]*cY[i];
      e[2][2]+=cY[i]*cY[i];
      e[3][2]+=cY[i]*cZ[i];
    }
  
  // Resoudre par pivot de Gauss

  int a,t,ligne,sol;
  float temp;
  for(i=0; i<3-1; i++){                
    for(a=1+i; a<3; a++){
      
      temp=e[i][a];
      for (t=i; t<3+1; t++){
	
	
	// On triangularise le systeme
	
	e[t][a] = e[t][a] * e[i][i] - e[t][i] * temp;
	
      }
      
    }
  }
  
  // On remplace
  s[3-1] = e[3][3-1] / e[3-1][3-1];
  e[3][3-1]=0;
  e[3-1][3-1]=0;
  
  for (ligne=1; ligne<=3; ligne++){
    for (sol=2; sol<=3; sol++){
      e[3-ligne][3-sol] *= s[3-ligne];
      e[3][3-sol] -= e[3-ligne][3-sol];
      e[3-ligne][3-sol]=0;
      
    }
    
    s[3-(ligne+1)] = e[3][3-(ligne+1)] / e[3-(ligne+1)][3-(ligne+1)];
    
  }
}
int cmp_float(const void *x, const void *y){
  float a=  *(float *)x;
  float b=  *(float *)y;
  return (int)(b - a);
}
int cmp_int(const void *x, const void *y){
  int a=  *(int *)x;
  int b=  *(int *)y;
  return (b - a);
}

float * doAlign(struct etoile * etoilesA,
	    struct etoile * etoilesB,
	    struct triangle * trianglesA,
	    struct triangle * trianglesB)
{
  // struct triangle * pt=trianglesA;
/*   while (pt!=NULL) */
/*     { */
/*       printf(" Tab Triangle %i %i %i : %f %f  \n", pt->iA,pt->iB,pt->iC,pt->xt,pt->yt); */
/*       pt=pt->suivant; */
/*     } */
  struct etoile * pe=etoilesA;
/*   while (pe!=NULL) */
/*     { */
/*       if (pe->indice!=-1) */
/* 	{ */
/* 	  printf(" Tab etoile %i %i %f %f   \n",  */
/* 		 pe->indice, */
/* 		 pe->intensite, */
/* 		 pe->centre_X, */
/* 		 pe->centre_X); */
/* 	} */
/*       pe=pe->suivant; */
/*     } */
  // Matching des triangles
  // et Vote
  int vote[NBOBJ][NBOBJ]; // tableau dynamique ?
  int v[6];
  int b;
  int iA[6];
  int iB[6];
  int ind;
  int i,j;
  float cX[6],cY[6],cZ1[6],cZ2[6];
  for (i=0;i<NBOBJ;i++)
    for (j=0;j<NBOBJ;j++)
      vote[i][j]=0;
  float d;
  float dd[6];
  float moy,moy2;
  float ep=EPSILON;
  struct triangle * ptA=trianglesA;
  struct triangle * ptB;
  //int da=0;
  //int db=0;
  int nbm=0;
  float s1[3];
  float s2[3];
  while (ptA!=NULL)
    {
      ptB=trianglesB;
      while (ptB!=NULL)
	{
	  d=pow(ptA->xt-ptB->xt,2)+pow(ptA->yt-ptB->yt,2);
	  //da++;
	  if (d<ep)
	    {
	      //db++;
	      nbm++;
/* 	      printf ("match A(%i,%i,%i) B(%i,%i,%i) \n", */
/* 		      ptA->iA,ptA->iB,ptA->iC, */
/* 		      ptB->iA,ptB->iB,ptB->iC); */
	      vote[ptA->iA][ptB->iA]+=1;
	      vote[ptA->iB][ptB->iB]+=1;
	      vote[ptA->iC][ptB->iC]+=1;
	      
	    }
	  
	  ptB=ptB->suivant;
	}
      
      ptA=ptA->suivant;
    }
  //printf("Nb de triangles qui match : %i\n",nbm);
  int nbBoucleMax=0;
  // debut analyse
  int fin=0;
  //printf ("Inération :"); 
  while ((fin==0)&&(nbBoucleMax<1500))
    {
      nbBoucleMax++;
      printf ("."); 

      fflush(stdout);
      //
      //vote[26][27]=0;
      /*  vote[2][0]=0; */
      /*    vote[2][1]=0;  */
      /*   vote[19][16]=0; */
      /*    vote[1][0]=0;  */
  
/*       for (i=0;i<NBOBJ;i++) { */
/* 	printf("\n"); */
/* 	for (j=0;j<NBOBJ;j++) */
/* 	  { */
/* 	    printf("%3i ",vote[i][j]); */
/* 	  }} */
/*       printf("\n"); */

      // Les six  meilleurs taux
      int valeurs[NBOBJ*NBOBJ];
      for (i=0;i<NBOBJ;i++) 
	for (j=0;j<NBOBJ;j++)
	  valeurs[i*NBOBJ+j]=vote[i][j];
      qsort(valeurs,NBOBJ*NBOBJ,sizeof(float),cmp_int);
      for (i=0;i<6;i++) v[i]=-1;
      i=0;
      b=0;
      int precedent=0;
      while ((b<6)&&(i<(NBOBJ*NBOBJ)))
	{
	  if (precedent != valeurs[i])
	    {
	      precedent = valeurs[i];
	      if (v[b]<valeurs[i]) 
		{
		  v[b]=valeurs[i];
		  b++;
		}
	    }
	  i++;
	}
      for (i=0;i<NBOBJ;i++) {
	for (j=0;j<NBOBJ;j++)
	  if (vote[i][j]!=-1)
	    {
	      if (vote[i][j]>=v[0]) 
		v[0]=vote[i][j];
	      if ((vote[i][j]>=v[1])&&(vote[i][j]<v[0]))
		v[1]=vote[i][j];
	      if ((vote[i][j]>=v[2])&&(vote[i][j]<v[1]))
		v[2]=vote[i][j];
	      if ((vote[i][j]>=v[3])&&(vote[i][j]<v[2]))
		v[3]=vote[i][j];
	      if ((vote[i][j]>=v[4])&&(vote[i][j]<v[3]))
		v[4]=vote[i][j];
	      if ((vote[i][j]>=v[5])&&(vote[i][j]<v[4]))
		v[5]=vote[i][j];
	    }
      }
      // On selectionne les 6 meilleurs couples
      for (i=0;i<6;i++)
	{
	  iA[i]=-1;
	  iB[i]=-1;
	}
      ind = 0;  
      for (b=0;b<6;b++)
	for (i=0;i<NBOBJ;i++) 
	  for (j=0;j<NBOBJ;j++)
	    if ((ind<6)&&(vote[i][j]==v[b]))
	      {
		iA[ind]=i;
		iB[ind]=j;
		ind++;
	      }

      // il reste assez de valeurs ?
      for (b=0;b<6;b++)
	{
	  if ((v[b]==-1)||(iA[b]==-1)||(iB[b]==-1))
	    {
	      fin=1; // Pas assez d'étoiles 
	      printf("Fin pas assez d'etoiles\n");
	      return NULL;
	    }
	 	 
	}
      
      for (b=0;b<6;b++)
	{
	  cX[b]=0.0;
	  cY[b]=0.0;
	  cZ1[b]=0.0;
	  cZ2[b]=0.0;
	  pe=etoilesA;
	  while (pe!=NULL)
	    {
	      if (pe->indice==iA[b])
		{
		  cX[b]=pe->centre_X;
		  cY[b]=pe->centre_Y;
		}
	      pe=pe->suivant;
	    }
	  pe=etoilesB;
	  while (pe!=NULL)
	    {
	      if (pe->indice==iB[b])
		{
		  cZ1[b]=pe->centre_X;
		  cZ2[b]=pe->centre_Y;
		}
	      pe=pe->suivant;
	    }
	  /* 	  printf ("v%i %i (%i, %i) %f %f %f\n ",b,v[0], */
	  /* 		  iA[b],iB[b], */
	  /* 		  cX[b],cY[b],cZ1[b]); */
	}
      
      int methode=1;

      if (methode==1)
	{

	  coefTransformation(cX,cY,cZ1,s1);
	  coefTransformation(cX,cY,cZ2,s2);
	}
      else
	{
	  s1[0]=coefTranslation(cX,cZ1);
	  s1[1]=1;
	  s1[2]=0;
	  s2[0]=coefTranslation(cY,cZ2);
	  s2[1]=0;
	  s2[2]=1;
	}

      // Valider les résultats
      float xp,yp;
      moy=0.0;
      moy2=0.0;
      float clip;
      for (i=0;i<6;i++)
	{
	  xp=s1[0]+s1[1]*cX[i]+s1[2]*cY[i];
	  yp=s2[0]+s2[1]*cX[i]+s2[2]*cY[i];
	  dd[i]=sqrt(pow(cZ1[i]-xp,2)+pow(cZ2[i]-yp,2));
	  moy +=dd[i];
	  moy2 +=dd[i]*dd[i];
	  /* 	  printf("%i (%f,%f) -> (%f,%f) = %f \n",i, */
	  /* 	  	 cZ1[i],cZ2[i],xp,yp,dd[i]); */
	}
      clip=moy/6 +(sqrt(moy2/6-pow(moy/6,2))*2*0.6);
      /*       printf("Moyenne : %f avec %f clip %f\n", */
      /* 	     moy/6, */
      /* 	     sqrt(moy2/6-pow(moy/6,2)), */
      /* 	     clip); */
      int methodeFin = 2;
      if  (methodeFin == 1)
	{
	  // Methode 1
	  fin=2; // Ca marche
	  //qsort( dd, 6, sizeof(float),
	  //cmp_float);    /* la fonction qui compare les elements */
	  for (i=0;i<6;i++)
	    {
	      if (dd[i]>clip)
		{
		  // reject
		  vote[iA[i]][iB[i]]=-1;
		  fin=0;
		}
	    }
	}
      else
	{
	  // Methode 2
	  if(sqrt(moy2/6-pow(moy/6,2))<1.0)
	    {
	      fin=2;
	    }
	  else
	    {
	      // On elimine la plus grande valeur
	      float valMax=0.0;
	      for (i=0;i<6;i++) if (valMax<=dd[i]) valMax=dd[i];
	      for (i=0;i<6;i++)
		if (dd[i]==valMax)
		  {
		    // reject
		    vote[iA[i]][iB[i]]=-1;
		  }
	    }
	}
    } 
    
  printf("\nFonction de tranformation:\n");
  printf("===========================\n\n");
  printf("x'=%f+%f*x+%f*y\n",s1[0],s1[1],s1[2]);
  printf("y'=%f+%f*x+%f*y\n\n",s2[0],s2[1],s2[2]);
  //printf("Moyenne %3.3f Ecart type:%3.3f\n",moy/6,sqrt(moy2/6-pow(moy/6,2)));

  float *trans;
  trans=malloc(sizeof(float)*7);
  trans[0]=s1[0];
  trans[1]=s1[1];
  trans[2]=s1[2];
  trans[3]=s2[0];
  trans[4]=s2[1];
  trans[5]=s2[2];
  trans[6]=sqrt(moy2/6-pow(moy/6,2));
  return trans;

}

int  doTrt2(int AvecP)
{
  struct image *offset;
  struct image * dark;
  struct image * flat;
  
  printf("Traitement partie 2\n");
  /*   Lire le repertoire des images */
  
  struct dirent *lecture;
  DIR *rep;
  // Si on fait le pretraitement en meme temps alors lire light
  if (AvecP) 
    {
      rep = opendir(REPERTOIRES[LIGHTS]);
      
      /* lire offset */
      offset=malloc(sizeof(struct image));
      loadTiff("results//offset.tiff", offset,A_TRUE);
      
      /* lire dark */
      dark=malloc(sizeof(struct image));
      loadTiff("results//dark.tiff", dark,A_TRUE);
      
      /* lire flat */
      flat=malloc(sizeof(struct image));
      loadTiff("results//flat.tiff", flat,A_TRUE);
    }
  else
    rep = opendir(REPERTOIRES[RESULTS]);
  char *name;

  struct etoile * etoilesA;
  struct etoile * etoilesB;
  struct triangle * trianglesA;
  struct triangle * trianglesB;
  struct image * resultat;

  float *trans;
  uint16 *R, *G, *B;
  struct image im;

  findSizeImage(&im);
  R = malloc(sizeof(int)*im.width*im.height);
  G = malloc(sizeof(int)*im.width*im.height);
  B = malloc(sizeof(int)*im.width*im.height); 

  int first=1; // La premiere image sert de base
  int iT,iR;
  iT=0;
  iR=0;

  /* lire les images */
  while ((lecture = readdir(rep))) {
    
    /* On prend que les fichiers .tiff */
    if  (strlen(lecture->d_name) > 5 )
      
      {
	char *p = lecture->d_name;
	p = p + strlen(lecture->d_name)-5;
	// Avec pretraitement, il faut lire les tiff existants sinon les .data
	char comp[6];
	if (AvecP) 
	  strcpy(comp,".tiff");
	else
	  strcpy(comp,".data");
	if (strcmp(p, comp)==0)
	  {
	    lecture->d_name[strlen(lecture->d_name)-5]='\0';

	    struct image * im;
	    im=malloc(sizeof(struct image));
	    // Si avev pretraitement alors on fait le pretraitement
	    // sinon on charfe l'image
	    if (AvecP)
	      {
		findSizeImage(im);
		im->sampleParPixel=3;
		im->bps=16;
		im->exposure=0;
		im->bufferSize = im->width * im->height * im->sampleParPixel *2;
		if((im->buffer = (char *) malloc(im->bufferSize)) == NULL){
		  fprintf(stderr, "Could not allocate enough memory for the uncompressed image\n");
		  exit(42);
		}
		// Intialisation du buffer image sortie
		uint32 count;
		for(count = 0; count < im->bufferSize  ; count++) im->buffer[count]=0;

		doTrt(lecture->d_name,im,offset,dark, flat);
	      }
	    else
	      {
		char fic[500]; // buffer dynamique a faire
		sprintf(fic,"%s//%s.tiff","results",lecture->d_name);
		loadTiff(fic, im,A_TRUE);
	      }

	    /* Traitement des images */
	    if (first) {
	      printf ("Image de base :\n");
	      iT++;
	      iR++;
	      first=0;
	      etoilesA = lectureFichier(lecture->d_name);
	      trianglesA = creationTriangles(etoilesA);
	      resultat = doAdditionA(im,R,G,B);
	    }
	    else {
	      printf ("Image a additionner\n");
	      iT++;
	      etoilesB = lectureFichier(lecture->d_name);
	      trianglesB=creationTriangles(etoilesB);
	      trans = doAlign(etoilesB,etoilesA,trianglesB,trianglesA); 
	      // Le septieme coef de trans est l'écart de la transformation si > 1 on rejete l'iamge
	      if ((trans!=NULL)&&(trans[6]<1.0))
		{
		  doAdditionB(im,trans,R,G,B,resultat);
		  iR++;
		}
	      else
		printf("L'image %s est rejectee ! <---------\n",lecture->d_name);
		
	    }
	    free(im->buffer);
	    free(im);
	    
	  }
      }
  }
  closedir(rep);
  doAdditionC("fin",R,G,B,resultat,iR);
  printf("\n %i/%i\n",iR,iT);
}

int  doTrt3(int zX, int zY , int zL, int zH,  int zZ)
{
  // Alignement avec zoom sur une portion d'image
  
  printf("Traitement partie 3\n");
  /*   Lire le repertoire des images */
  
  struct dirent *lecture;
  DIR *rep;
  rep = opendir("results");
  char *name;

  struct etoile * etoilesA;
  struct etoile * etoilesB;
  struct triangle * trianglesA;
  struct triangle * trianglesB;
  struct image * resultat;

  float *trans;
  uint16 *R, *G, *B;
  float *tP;
  float tPmax;
  
  R = malloc(sizeof(int)*zL*zH*zZ*zZ);
  G = malloc(sizeof(int)*zL*zH*zZ*zZ);
  B = malloc(sizeof(int)*zL*zH*zZ*zZ);
  tP = malloc(sizeof(float)*zL*zH*zZ*zZ); 

  //printf("%i %i %i %i %i %i\n",zX, zY , zL, zH,  zZ,zL*zH*zZ);fflush(stdout);
  
  int first=1; // La premiere image sert de base
  int iT,iR;
  iT=0;
  iR=0;

  /* lire les images */
  while ((lecture = readdir(rep))) {
    
    /* On prend que les fichiers .tiff */
    if  (strlen(lecture->d_name) > 5 )
      
      {
	char *p = lecture->d_name;
	p = p + strlen(lecture->d_name)-5;
	if (strcmp(p, ".data")==0)
	  {
	    lecture->d_name[strlen(lecture->d_name)-5]='\0';
	    /* Traitement des images */
	    if (first) {
	      //printf ("premier\n");
	      first=0;
	      iT++;
	      iR++;
	      etoilesA = lectureFichier(lecture->d_name);
	      trianglesA = creationTriangles(etoilesA);
	      resultat = doZoomAdditionA(lecture->d_name,R,G,B,
					 zX,zY,zL,zH,zZ,tP,&tPmax);
	    }
	    else {
	      //printf ("deuz\n");
	      iT++;
	      etoilesB = lectureFichier(lecture->d_name);
	      trianglesB=creationTriangles(etoilesB);
	      trans = doAlign(etoilesB,etoilesA,trianglesB,trianglesA); 
	      // Le septieme coef de trans est l'écart de la transformation si > 1 on rejete l'iamge
	      if ((trans!=NULL)&&(trans[6]<1.0))
		{
		  doZoomAdditionB(lecture->d_name,trans,R,G,B,resultat,
				  zX,zY,zL,zH,zZ,tP,&tPmax);
		  iR++;
		}
	      else
		printf("L'image %s est rejectee ! <---------\n",lecture->d_name);
		
	    }
	    
	  }
      }
  }
  
  closedir(rep);
  doZoomAdditionC("zoom",R,G,B,resultat,tP,tPmax,iR);
  //printf("\n %i/%i\n",iR,iT);
}

copy16to8II(struct image * Isrc,struct image * gen, int seuil_b,int seuil_h)
{
  printf("Conversion Tiff 16 b vers 8 :\n");
  fflush(stdout);

  copyImage(Isrc, gen);
  gen->sampleParPixel=3;
  gen->bps=8;
  gen->bufferSize = gen->width * gen->height * gen->sampleParPixel ;
  free(gen->buffer);
  if((gen->buffer = (char *) malloc(gen->bufferSize)) == NULL){
    fprintf(stderr, "Could not allocate enough memory for the uncompressed image\n");
    exit(42);
  }

  int i,j,x,y;
  uint16 R,G,B;
  float r,g,b;
  R=0;G=0;B=0;
  // Recherche de la fonction de recherche d'une valeur d'un pixel
  _ImVal ImVal  = DefImVal(Isrc);
  _ImMaj ImMaj  = DefImMaj(gen);

  float t;
  t=255.0/(float)(seuil_h-seuil_b);

  for (y=0;y<gen->height;y++)
    for (x=0;x<gen->width;x++)
      {
	(ImVal)(Isrc,x,y,&R,&G,&B);
 
	r = ((float)R-(float)seuil_b)*t;
	g = ((float)G-(float)seuil_b)*t;
	b = ((float)B-(float)seuil_b)*t;
	R = R<seuil_b?0:R>seuil_h?255:(uint16)r;
	G = G<seuil_b?0:G>seuil_h?255:(uint16)g;
	B = B<seuil_b?0:B>seuil_h?255:(uint16)b;

	(ImMaj)(gen,x,y,R,G,B);	
      }
}

void copy16to8IF(struct image * Isrc,char *dst, int seuil_b,int seuil_h)
{
  struct image * gen;
  gen = malloc(sizeof(struct image));

  copy16to8II(Isrc, gen, seuil_b,seuil_h);

  /*   Ecrire l'image sur disk */
  writeTiff(gen,dst);
  free(gen->buffer);  
}

void copy16to8(char *src,char *dst, int seuil_b,int seuil_h)
{

  /* lire la src */
  struct image Isrc;
  loadTiff(src, &Isrc,A_TRUE);
  
  copy16to8IF(&Isrc,dst,seuil_b,seuil_h);
  
  free(Isrc.buffer);
  printf("Ok\n");
  fflush(stdout);

  
}

int makeStack(int typ_traitment)
{
  char resultat[256];
 
  uint16 R,G,B;
  uint16 oR,oG,oB;
  uint16 dR,dG,dB;
  oR=0; oG=0; oB=0;
  dR=0; dG=0; dB=0;
 
  // Pour les darks et flats
  struct image offset;
  struct image dark;
  _ImVal ImValOff;
  _ImVal ImValDrk;
 

  switch (typ_traitment)
      {
      case OFFSETS:
	printf("Creation de l'offset :\n");
	fflush(stdout);
	sprintf(resultat,"results//offset.tiff");
	R=0; G=0; B=0;
	break;
      case DARKS:
	printf("Creation du dark :\n");
	fflush(stdout);
	sprintf(resultat,"results//dark.tiff");
	loadTiff("results//offset.tiff", &offset,A_TRUE);
	R=0; G=0; B=0;
	ImValOff = DefImVal(&offset); 
	break;
      case FLATS:
	printf("Creation du flat :\n");
	fflush(stdout);
	sprintf(resultat,"results//flat.tiff");
	loadTiff("results//offset.tiff", &offset,A_TRUE);
	loadTiff("results//dark.tiff", &dark,A_TRUE);
	R=1; G=1; B=1;
	ImValOff = DefImVal(&offset);
	ImValOff = DefImVal(&dark);
	break;
      default:
	printf("Type de traitement non implemente :\n");
	fflush(stdout);
	return -1;
	break;
      }

  /* A faire  tableau dynamique */
  /* Lire le nombre d'images si plus grnad que 16 alors pas possible ...*/
  struct image ** images;
  images = malloc(sizeof(struct image)*16);
  int i=0;

  i = lireImageRep(REPERTOIRES[typ_traitment], images);
  
  /*   Additionner les buffers */
  int j;
  int v;
  unsigned long count;
  
  struct image stack;

 
  int x,y;

  if (i==0)
    {
      // Pas d'image
      findSizeImage(&stack);
      stack.sampleParPixel=1;
      stack.bps=16;
      stack.exposure=0;
      stack.bufferSize = stack.width * stack.height * stack.sampleParPixel *2;
      if((stack.buffer = (char *) malloc(stack.bufferSize)) == NULL){
	fprintf(stderr, "Could not allocate enough memory for the uncompressed image\n");
	exit(42);
      }
      _ImMaj ImMaj  = DefImMaj(&stack);
      for (y=0;y<stack.height;y++)
	for (x=0;x<stack.width;x++)
	  (ImMaj)(&stack,x,y,R,G,B);
    }
  else
    {
      
      uint16 m;
      
      // Empilement par mediane
      // Si i est pair, on prend la valeur i/2 sinon
      // on prend la moyenne entre valeur i/2 et i/2 +1

      copyImage(images[0],&stack);
      _ImMaj ImMaj  = DefImMaj(&stack);
      
      int med = (int)i/2;
      int pair = i - med*2; // 0 si pair 1 si impair
      
      printf("Methode par Mediane\n"); fflush(stdout);
      
      int t[16];
      _ImVal ImVal[16];

      // Initialisation des fonctions de lecture des images
      for (j=0;j<i;j++)
	{
	  ImVal[j] = DefImVal(images[j]);
	}

      // Boucle sur les pixels de l'image

      for (y=0;y<stack.height;y++)
	for (x=0;x<stack.width;x++)
	  {
	    // Recherches des valeurs necessaires
	    switch (typ_traitment)
	      {
	      case DARKS:
		(ImValOff)(&offset,x,y,&oR,&oG,&oB);
		break;
	      case FLATS:
		(ImValOff)(&offset,x,y,&oR,&oG,&oB);
		(ImValOff)(&dark,x,y,&dR,&dG,&dB);
		break;
	      }
	    
	    
	    // Mise en tableau des valeurs de pixel
	    for (j=0;j<i;j++)
	      {
		// Valeur
		(ImVal[j])(images[j],x,y,&R,&G,&B);
		switch (typ_traitment)
		  {
		  case OFFSETS:
		    v = R + SEUIL; // Un seuil pour eviter d'etre a zero
		    break;
		  case DARKS:
		    v = R + SEUIL_OFFSET -oR;
		    v=v<0?0:v;
		    break;
		  case FLATS:
		    v = R + SEUIL_OFFSET -oR -dR;
		    v=v<=0?1:v;
		    break;
		  }
		
		t[j]=v;
	      }
	    // tri des valeurs
	    qsort( &t,
		   i, /* le nombre d'elemnts */
		   sizeof(int), /* la taille d'un element */
		   cmp_int);    /* la fonction qui compare les elements */
	    
	    
	    // Choix de la valeur mediane
	    m = (t[med]+t[med+pair])/2;
	    
	    // Stockage de la mediane
	    (ImMaj)(&stack,x,y,m,G,B);
	  }
    }

  /*   Ecrire l'image sur disk */
  writeTiff(&stack,resultat);
  free(images);
  printf("Ok\n");
  fflush(stdout);
}

void makeFlat()
{
  makeStack(FLATS);
}

void makeDark()
{
  makeStack(DARKS);
}

void makeOffset()
{
  makeStack(OFFSETS);
}


void doAnalyseRep()
{
  int i;
  struct image im ;
  for(i=0;i<LEN_REPERTOIRES;i++)
    {

      printf ("%s\n",REPERTOIRES[i]);
      // Lecture d'un repertoire 
     
      struct dirent *lecture;
      DIR *rep;
      rep = opendir(REPERTOIRES[i]);
      
      if (rep)

	/* lire les images */
	while ((lecture = readdir(rep))) {
	  /* On prend que les fichiers qui ne commence pas par . */
	  if  ((strlen(lecture->d_name) > 1 )&&
	       (lecture->d_name[0] != '.' ))
	    
	    {
	      // Fichier TIFF
	      char *p = lecture->d_name;
	      p = p + strlen(lecture->d_name)-5;
	      if (strcmp(p, ".tiff")==0)
		{
		  char path[512]; // Buffer dynamique a faire
		  sprintf(path,"%s//%s",REPERTOIRES[i],lecture->d_name);
		  if (loadTiff(path, &im,A_FALSE))
		    
		    {
		      printf ("%s %ix%i %i bps %i spp %f \n",
			      path,
			      im.width,
			      im.height,
			      im.bps,
			      im.sampleParPixel,
			      im.exposure);
		      fflush(stdout);
		      free(im.buffer);
		    }
		  else
		    printf ("%s\n",path);
		}
	      
	    }
	}
      closedir(rep);
      
    }
}


void cleanRep()
{
  int i;
  struct image im ;
  for(i=0;i<(LEN_REPERTOIRES);i++)
    {

      printf ("%s\n",REPERTOIRES[i]);
      // Lecture d'un repertoire 
     
      struct dirent *lecture;
      DIR *rep;
      rep = opendir(REPERTOIRES[i]);
      
      if (rep)

	/* lire les images */
	while ((lecture = readdir(rep))) {
	  /* On prend que les fichiers qui ne commence pas par . */
	  if  ((strlen(lecture->d_name) > 1 )&&
	       (lecture->d_name[0] != '.' ))
	    
	    {
	      if (i!=RESULTS) {
		// Fichier TIFF
		char *p = lecture->d_name;
		p = p + strlen(lecture->d_name)-5;
		if (strcmp(p, ".tiff")==0)
		  {
		    char path[512]; // Buffer dynamique a faire
		    sprintf(path,"%s//%s",REPERTOIRES[i],lecture->d_name);
		    if (loadTiff(path, &im,A_FALSE))
		      
		      {
			printf ("%s %ix%i %i bps %i spp %f \n",
				path,
				im.width,
				im.height,
				im.bps,
				im.sampleParPixel,
				im.exposure);
			fflush(stdout);
			free(im.buffer);
			remove(path);
		      }
		  }
	      }
	      if (i==RESULTS) {
		// Fichier data
		char *p = lecture->d_name;
		p = p + strlen(lecture->d_name)-5;
		if (strcmp(p, ".data")==0)
		  {
		    char path[512]; // Buffer dynamique a faire
		    sprintf(path,"%s//%s",REPERTOIRES[i],lecture->d_name);
		    printf ("%s \n",path);
		    fflush(stdout);
		    remove(path);
		    
		  }
	      }
	    }
	}
      closedir(rep);
      
    }
}

void doPretraitement()
{
  struct image *offset;
  struct image * dark;
  struct image * flat;

  /* lire offset */
  offset=malloc(sizeof(struct image));
  loadTiff("results//offset.tiff", offset,A_TRUE);
  
  /* lire dark */
  dark=malloc(sizeof(struct image));
  loadTiff("results//dark.tiff", dark,A_TRUE);
	
  /* lire flat */
  flat=malloc(sizeof(struct image));
  loadTiff("results//flat.tiff", flat,A_TRUE);
  
  struct dirent *lecture;
  DIR *rep;
  rep = opendir("lights");
  char *name;
  /* lire les images */
  while ((lecture = readdir(rep))) {
    /* On prend que les fichiers .tiff */
    if  (strlen(lecture->d_name) > 5 )
      
      {
	char *p = lecture->d_name;
	p = p + strlen(lecture->d_name)-5;
	if (strcmp(p, ".tiff")==0)
	  {
	    lecture->d_name[strlen(lecture->d_name)-5]='\0';
	    printf("Traitement de %s :\n",lecture->d_name);
	    fflush(stdout);
	    /* Traitement des images */
	    struct image * gen;
	    gen = malloc(sizeof(struct image));
	    findSizeImage(gen);
	    gen->sampleParPixel=3;
	    gen->bps=16;
	    gen->exposure=0;
	    gen->bufferSize = gen->width * gen->height * gen->sampleParPixel *2;
	    if((gen->buffer = (char *) malloc(gen->bufferSize)) == NULL){
	      fprintf(stderr, "Could not allocate enough memory for the uncompressed image\n");
	      exit(42);
	    }
	    // Intialisation du buffer image sortie
	    uint32 count;
	    for(count = 0; count < gen->bufferSize  ; count++) gen->buffer[count]=0;

	    doTrt(lecture->d_name,gen,offset,dark,flat); 

	    char fic[500]; // buffer dynamique a faire
	    sprintf(fic,"%s//%s.tiff","results",lecture->d_name);
	    writeTiff(gen, fic);
	    free(gen->buffer);
	    free(gen);
	  }
      }
  }
  closedir(rep);

}

// LRGB Prise de notes
// RGB Luminance value   =   0.3 R + 0.59 G + 0.11 B
// http://www.tsi.enst.fr/tsi/enseignement/ressources/mti/RVB_ou_LAB/html/colorspace.html

void doPA()
{

  doTrt2(A_TRUE);

}

void doThumbnail(char *src,char *dst, int seuil_b,int seuil_h,int larg_souhaite, int haut_souhaite)
{
  printf("Creation d'une petite image :\n");
  fflush(stdout);

  /* lire la src */
  struct image Isrc;
  loadTiff(src, &Isrc,A_TRUE);
  
  // Calcul des largeurs et hauteurs de l'imagette
  int larg;
  int haut;
  float c1=(float)larg_souhaite/Isrc.width;
  float c2=(float)haut_souhaite/Isrc.height;
  if (c1<c2) c2=c1;
  larg = Isrc.width*c2;
  haut = Isrc.height*c2;

  printf("%i %i\n", larg,haut);
  
  struct image gen;
  gen.width = larg;
  gen.height =  haut;
  gen.sampleParPixel=3;
  gen.bps=8;
  gen.exposure=Isrc.exposure;
  gen.bufferSize = gen.width * gen.height * gen.sampleParPixel ;
  gen.buffer;
  if((gen.buffer = (char *) malloc(gen.bufferSize)) == NULL){
    fprintf(stderr, "Could not allocate enough memory for the uncompressed image\n");
    exit(42);
  }

  unsigned long genCount,IsrcCount,x,y;
  unsigned int Rv,Gv,Bv;
  Rv=0;
  Gv=0;
  Bv=0;
  float t=255.0/(float)seuil_h;
  float Xcoef=Isrc.width/gen.width;
  float Ycoef=Isrc.height/gen.height;
  int X,Y;
  
  for (y=0;y<gen.height;y++)
    for (x=0;x<gen.width;x++)
      {
	genCount = y*gen.width+ x;
	X=(int)x*Xcoef;
	Y=(int)y*Ycoef;

	IsrcCount = Y*Isrc.width+ X;
	if (Isrc.sampleParPixel==1)
	  if (Isrc.bps==8)
	    // 1 couche 8 bits
	    {
	      Rv=Isrc.buffer[IsrcCount];
	      Gv=Rv;
	      Bv=Rv;
	    }
	  else
	    // 1 couche 16 bits
	    {
	      Rv=Isrc.buffer[IsrcCount*2]+256*Isrc.buffer[IsrcCount*2+1];
	      Gv=Rv;
	      Bv=Rv;
	    }
	else
	  if (Isrc.bps==8)
	    // 3 couche 8 bits
	    {
	      Rv=Isrc.buffer[IsrcCount*3];
	      Gv=Isrc.buffer[IsrcCount*3+1];
	      Bv=Isrc.buffer[IsrcCount*3+2];
	    }
	  else
	    // 3 couche 16 bits
	    {
	      Rv=Isrc.buffer[IsrcCount*6]+256*Isrc.buffer[IsrcCount*6+1];
	      Gv=Isrc.buffer[IsrcCount*6+2]+256*Isrc.buffer[IsrcCount*6+3];
	      Bv=Isrc.buffer[IsrcCount*6+4]+256*Isrc.buffer[IsrcCount*6+5];
	    }


	if (Rv>seuil_h) gen.buffer[genCount*3]=255;
	else  gen.buffer[genCount*3]=(unsigned int)(Rv*t);
	
	if (Gv>seuil_h) gen.buffer[genCount*3+1]=255;
	else gen.buffer[genCount*3+1]=(unsigned int)(Gv*t);
	
	if (Bv>seuil_h) gen.buffer[genCount*3+2]=255;
	else  gen.buffer[genCount*3+2]=(unsigned int)(Bv*t);
	
      }


  /*   Ecrire l'image sur disk */
  writeTiff(&gen,dst);
  free(gen.buffer);  
  free(Isrc.buffer);
  printf("Ok\n");
  fflush(stdout);
  
}

