/*
   main.c -- APA 
   Copyright 2007 by David Laine, david dot laine a free o fr

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
   
 */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <locale.h>

#include "apa.h"
#include "../config.h"

int convertRAW()
{
  int Sargc;
  char* Sargv[5];
  // Simulation d'un appel à DCRAW
  Sargv[0]="DCRAW";
  Sargv[1]="-D";
  Sargv[2]="-T";
  Sargv[3]="-4";
  Sargv[4]=malloc(sizeof(char)*512);

  int i;
  for(i=0;i<LEN_REPERTOIRES;i++)
    {

      // Lecture d'un repertoire 
     
      struct dirent *lecture;
      DIR *rep;
      rep = opendir(REPERTOIRES[i]);
  
      /* lire les images */
      while ((lecture = readdir(rep))) {
	/* On prend que les fichiers .CR2 */
	if  (strlen(lecture->d_name) > 4 )
	  
	  {
	    char *p = lecture->d_name;
	    p = p + strlen(lecture->d_name)-4;
	    if ((strcmp(p, ".CR2")==0)||
		(strcmp(p, ".CRW")==0))
	      {
		char nRAW[512];
		sprintf(nRAW,"%s//%s",REPERTOIRES[i],lecture->d_name);
		printf("RAW en Tiff : %s\n", nRAW);
		fflush(stdout);
		
		strcpy(Sargv[4],nRAW);
		//  ./dcraw.exe -D -T -4 lights/IMG_1895.CR2
		main2(5,Sargv);
		
		
	      }
	  }
      }
      closedir(rep);
      
   }
 
}



int usage(char *pgm)
{
  wchar_t *s = L"[OPTION]\n"
    L"\n\n"
    L"  -v     \t Version\n"
    L"  -t     \t Generation des tiffs a partir des RAW\n"
    L"  -o     \t Generation de l offset\n"
    L"  -d     \t Generation du dark\n"
    L"  -f     \t Generation du flat\n"
    L"  -D     \t Option p et a sans ectiture des fichiers intermediaires\n"
    L"  -p     \t Pretraitement\n"
    L"  -a     \t Alignement\n"
    L"  -r     \t Analyse des repertoires\n"
    L"  -R     \t Creation des repertoires vides sous Windows\n"
    L"  -s IMAGE    \t Statistique du fichier Tiff IMAGE \n"
    L"  -c     \t Transforme un image 16 bits vers 8 bits \n Les parametres sont source destination seuil_bas seuil_haut \n Par exemple apa -c src.tiff dst.tiff 0 1000\n"
    //L"  -z     \t Alignement d'une portion d'image avec zoom X Y Larg Haut Zoom \n"
    //L"  -H IMAGE   \t Histogramme \n"
    //L"  -T IMAGE   \t Creation d'une imagette \xe0 partir d'un fichier tiff \n"
    L"  -n     \t Nettoyage des repertoires intermediaires (supp des tiffs inuties)\n"
    L"  -e     \t Extraction des valeurs de pixels d'une zone delimitee par les point A et B ex: apa -e IMAGE 0x0 20x30\n"
    L"  -?     \t this help\n";
  printf("Usage: %s %ls\n",pgm,s);
  return 1;
}

int main(int argc, char* argv[])
{

  clock_t start, end;
  double elapsed;

  start = clock();	/* Lancement de la mesure */


  int c;
  extern int optind;
  extern char* optarg;
  if (argc ==1) return usage(argv[0]);
  
  int s ;

  setlocale(LC_CTYPE, "");
  
  while ((c = getopt(argc, argv, "vodfDrRpae:::s:ctH:?T:zn")) != -1)
    
    
    switch (c) 
      {
      case 'v':
	printf("\nVersion de %s : %s\n\n",argv[0],VERSION);
	exit(0);
	break;

      case 'R':
	makeRep();
	break;
      case 'D':
	doPA();
	break;
      case 't':
	convertRAW(); 
	break;
      case 'o':
	makeOffset(); 
	break;
      case 'd':
	makeDark(); 
	break;
      case 'f':
	makeFlat(); 
	break;
      case 'H':
	makeHistogramme(optarg); 
	break;
      case 'T':
	doThumbnail(optarg,"imagette.tiff", 0,500,500,500);
	break;
      case 'p':

	// Pré traitement
	doPretraitement();

	break;
      case 'r':
	doAnalyseRep();
	break;
	
      case 'a':
	doTrt2(A_FALSE);
	break;
      case 's':
	statTiff(optarg);
	break;
      case 'c':
	/*  */
	s= atoi(argv[4]);
	copy16to8(argv[2],argv[3], atoi(argv[4]), atoi(argv[5]));
	break;

      case 'z':
	doTrt3(atoi(argv[2]),atoi(argv[3]),atoi(argv[4]),atoi(argv[5]),atoi(argv[6]));
	break;
      case 'e':
	if (argc!=5) 
	  usage(argv[0]);
	else
	  extractTiffGnuPlot(argv[2],argv[3],argv[4]);
	break;
      case 'n':
	cleanRep();
	break;
      case '?':
      default :
	usage(argv[0]);
      }
  
  int i;
  end = clock();		/* Arret de la mesure */
  elapsed = ((double)end - start) / CLOCKS_PER_SEC; /* Conversion en seconde */
  printf("Temps execution %f\n",elapsed);


  exit(0);
}
